SUBROUTINE ESFIPIN(FILEIN,FILEOUT)
!-----------------------------------------------------------------------------
!
! READS AN ASEG ESF FORMAT IP FILE AND WRITES TO A BINARY DATABASE
! NB. THIS ROUTINE IS  NOT A GENERAL ESF READER, IT ONLY CONCENTRATES ON THOSE 
! KEYWORDS PERTAINING TO IP SURVEYS. A SEPARATE BUT SIMILAR ROUTINE EXISTS FOR EM DATA.
! ALLOWS FOR HISTORICAL KEYWORDS BUT ANY EXPORT CODE SHOULD USE THE PREFERRED KEYWORDS
! NOT THE HISTORICAL ONES. 
!
! kmf 08/15
!------------------------------------------------------------------------------
USE INPPARAMS
USE WINTERACTER                                       !PROVIDES WINDOWS GUI

IMPLICIT NONE
CHARACTER(LEN=255), INTENT(IN) :: FILEIN              ! FILENAME FOR INPUT DATA FILE
CHARACTER(LEN=255), INTENT(IN) :: FILEOUT             ! FILENAME FOR OUTPUT DATABASE FILE

INTEGER :: LUHIN                                      ! HANDLE FOR INPUT DATA FILE
INTEGER :: IFSTA,IFLIN                                ! FIELDS FOR STATION AND LINE
INTEGER :: JLINE                                      ! COUNTER FOR CURRENT LINE IN INPUT FILE
INTEGER :: MODE
INTEGER, PARAMETER :: IIP1=11                         ! OUTPUT DATABASE FIELD FOR WINDOW 1
INTEGER :: IC1X                                       ! OUTPUT DATABASE FIELD FOR CURRENT ELECTRODE 1 X
INTEGER :: IC1Y                                       ! OUTPUT DATABASE FIELD FOR CURRENT ELECTRODE 1 Y
INTEGER :: IC1Z                                       ! OUTPUT DATABASE FIELD FOR CURRENT ELECTRODE 1 Z
INTEGER :: IC2X                                       ! OUTPUT DATABASE FIELD FOR CURRENT ELECTRODE 2 X
INTEGER :: IC2Y                                       ! OUTPUT DATABASE FIELD FOR CURRENT ELECTRODE 2 Y
INTEGER :: IC2Z                                       ! OUTPUT DATABASE FIELD FOR CURRENT ELECTRODE 2 Z
INTEGER :: IP1X                                       ! OUTPUT DATABASE FIELD FOR POTENTIAL ELECTRODE 1 X
INTEGER :: IP1Y                                       ! OUTPUT DATABASE FIELD FOR POTENTIAL ELECTRODE 1 Y
INTEGER :: IP1Z                                       ! OUTPUT DATABASE FIELD FOR POTENTIAL ELECTRODE 1 Z
INTEGER :: IP2X                                       ! OUTPUT DATABASE FIELD FOR POTENTIAL ELECTRODE 2 X
INTEGER :: IP2Y                                       ! OUTPUT DATABASE FIELD FOR POTENTIAL ELECTRODE 2 Y
INTEGER :: IP2Z                                       ! OUTPUT DATABASE FIELD FOR POTENTIAL POTENTIAL ELECTRODE 2 Z
INTEGER :: ISTDDEV                                    ! OUTPUT DATABASE FIELD FOR POTENTIAL STANDARD DEVIATION IN READING
INTEGER :: ISP                                        ! OUTPUT DATABASE FIELD FOR POTENTIAL SP
INTEGER :: INSTAK                                     ! OUTPUT DATABASE FIELD FOR POTENTIAL NUMBER OF STACKS
INTEGER :: ICR                                        ! OUTPUT DATABASE FIELD FOR POTENTIAL CONTACT RESISTANCE
INTEGER :: IEND                                       ! LAST FIELD USED
INTEGER :: IRXDIP,ITXDIP                              ! INPUT FIELD FOR RX AND TX DIPOLE SIZE
INTEGER :: IFLDUSEF                                   ! OUTPUT DATABASE FIELD FOR USE FLAG
INTEGER :: JEXTRA                                     ! NUMBER OF EXTRA FIELDS
INTEGER :: NWINH                                      ! NUMBER OF WINDOWS DECLARED IN HEADER
INTEGER :: IERR
INTEGER, PARAMETER :: MAXEXTRAF = MAXFLDS-30          ! MAXIMUM NUMBER OF EXTRA FIELDS TO CATER FOR
INTEGER, PARAMETER :: MAXINFLDS=5*MAXFLDS             ! MAXIMUM NUMBER OF FIELDS IN INPUT FILE
INTEGER, DIMENSION(MAXINFLDS) :: JACOL                ! ARRAY TO KEEP TRACK OF WHAT IS STORED IN WHAT COLUMN IN ASCII FILE
INTEGER, DIMENSION(MAXEXTRAF) :: IFLDEXTRA            ! HOLDERS FOR EXTRA FIELDS
REAL :: RLINE                                         ! LINE NUMBER
REAL :: RCURR                                         ! TX CURRENT IF A CONSTANT
REAL :: P1Y, P1Z                                      ! CONSTANT VALUE FOR P1Y, P1Z
REAL :: P2X, P2Y, P2Z                                 ! CONSTANT VALUE FOR P2X, P2Y, P2Z
REAL :: ONTIME, OFFTIME                               ! TX ON TIME AND OFF TIME
CHARACTER(LEN=500) :: OUTMESS
CHARACTER(LEN=255) :: CHFNAME                         ! FILE NAME FOR CHANNEL FILE, IF GIVEN
CHARACTER(LEN=20)  :: NULLSTR                         ! NULL VALUE AS A STRING
CHARACTER(LEN=12), DIMENSION(MAXEXTRAF) :: LABEXTRA   ! LABELS FOR EXTRA FIELDS
CHARACTER(LEN=12), PARAMETER :: LABELSTDD='STD_DEV'   ! LABEL FOR STD DEV
CHARACTER(LEN=12), PARAMETER :: LABELSP='SP'          ! LABEL FOR SP
CHARACTER(LEN=12), PARAMETER :: LABELCX1='C1_EAST'    ! LABELS FOR CURRENT ELECTRODE 1 X,Y & Z
CHARACTER(LEN=12), PARAMETER :: LABELCY1='C1_NORTH'
CHARACTER(LEN=12), PARAMETER :: LABELCZ1='C1_ELEV'
CHARACTER(LEN=12), PARAMETER :: LABELCX2='C2_EAST'    ! LABELS FOR CURRENT ELECTRODE 2 X,Y & Z
CHARACTER(LEN=12), PARAMETER :: LABELCY2='C2_NORTH'
CHARACTER(LEN=12), PARAMETER :: LABELCZ2='C2_ELEV'
CHARACTER(LEN=12), PARAMETER :: LABELPX1='P1_EAST'    ! LABELS FOR POTENTIAL ELECTRODE 1 X,Y & Z
CHARACTER(LEN=12), PARAMETER :: LABELPY1='P1_NORTH'
CHARACTER(LEN=12), PARAMETER :: LABELPZ1='P1_ELEV'
CHARACTER(LEN=12), PARAMETER :: LABELPX2='P2_EAST'    ! LABELS FOR POTENTIAL ELECTRODE 1 X,Y & Z
CHARACTER(LEN=12), PARAMETER :: LABELPY2='P2_NORTH'
CHARACTER(LEN=12), PARAMETER :: LABELPZ2='P2_ELEV'
CHARACTER(LEN=20), PARAMETER :: OUTSUB = 'ESFIPIN'
LOGICAL :: LEOF
LOGICAL :: LWARNARR                                   !FLAG FOR FIRST ARRAY WARNING
LOGICAL :: LWARNLABEL                                 !FLAG FOR FIRST LABEL WARNING
LOGICAL :: LWARNPARSE                                 !FLAG FOR FIRST PARSING WARNING


! INITIALISE VARIABLES
LWARNARR=.TRUE.
LWARNLABEL=.TRUE.
LWARNPARSE=.TRUE.
LZONGE=.FALSE.
LDHOLE=.FALSE.
LSPECTRAL=.FALSE.
LTDOMAIN=.FALSE.
WTIME=0.0
LEOF=.FALSE.
ZDATA=ZNULL
JLINE=0
HARR(5:6)=ZNULL
HARR(12:18)=ZNULL
HSTR(8)=''
HSTR(10)=''
JACOL=-1
IEND=-1
ISTDDEV=-1
ISP=-1
INSTAK=-1
ICR=-1
IFLDEXTRA=-1
JEXTRA=0
NWIN=0
NHARM=0
CHFNAME=''
RCURR=ZNULL
RLINE=ZNULL
NULLSTR=''
NWINH=0
P1Y=ZNULL
P1Z=ZNULL
P2X=ZNULL
P2Y=ZNULL
P2Z=ZNULL
ONTIME=ZNULL
OFFTIME=ZNULL
IRXDIP=0
ITXDIP=0

WRITE(LUHPRT,'(/,"_______  LOADING AN ASEG ESF IP FILE  _______",//,"INPUT FILE IS ",A,//)')TRIM(FILEIN)
!
! OPEN FILE AND READ THE HEADER BLOCK TO DETERMINE ARRAY TYPE AND THUS FORMAT FOR REMAINDER OF DATA
CALL GETHANDL(LUHIN)                                      !ROUTINE TO GET THE NEXT FREE FILE HANDLE
OPEN(UNIT=LUHIN,FILE=FILEIN,ACCESS='SEQUENTIAL',FORM='FORMATTED',STATUS='OLD')
CALL RDESFHDR()
!
! OPEN THE OUTPUT FILE AND INITIALISE PARAMETERS
MODE=1 
CALL INITBDB(FILEOUT)
IF (INDEX(HSTR(10),'E').NE.0) THEN
  IFSTA=3
  IFLIN=4
ELSEIF (INDEX(HSTR(10),'N').NE.0) THEN
  IFSTA=4
  IFLIN=3
ELSE
  IFSTA=0
  IFLIN=0
END IF
!
! READ IN STATION DATA
CALL READESF()
! FLUSH DATABASE BUFFER
CALL BDBWRB(LUHOUT,0,ZDATA,NFLD)
!
! UPDATE FILE HEADER
MODE=2
LABEL(IC1X)=LABELCX1
LABEL(IC1Y)=LABELCY1
LABEL(IC1Z)=LABELCZ1
LABEL(IC2X)=LABELCX2
LABEL(IC2Y)=LABELCY2
LABEL(IC2Z)=LABELCZ2
LABEL(IP1X)=LABELPX1
LABEL(IP1Y)=LABELPY1
LABEL(IP1Z)=LABELPZ1
LABEL(IP2X)=LABELPX2
LABEL(IP2Y)=LABELPY2
LABEL(IP2Z)=LABELPZ2
IF (ISTDDEV > 0) LABEL(ISTDDEV)=LABELSTDD
IF (ISP > 0) LABEL(ISP)=LABELSP
!LABEL(IDUMN)=LABELDUM
IF (TRIM(HSTR(6)) == 'GRADIENT') THEN
  IF (ZMAX(IC1X)-ZMIN(IC1X) < 1.0E-5) THEN
    HARR(13)=ZMAX(IC1X)
  END IF
  IF (ZMAX(IC1Y)-ZMIN(IC1Y) < 1.0E-5) THEN
    HARR(14)=ZMAX(IC1Y)
  END IF
  IF (ZMAX(IC1Z)-ZMIN(IC1Z) < 1.0E-5) THEN
    HARR(15)=ZMAX(IC1Z)
  END IF
  IF (ZMAX(IC2X)-ZMIN(IC2X) < 1.0E-5) THEN
    HARR(16)=ZMAX(IC2X)
  END IF
  IF (ZMAX(IC2Y)-ZMIN(IC2Y) < 1.0E-5) THEN
    HARR(17)=ZMAX(IC2Y)
  END IF
  IF (ZMAX(IC2Z)-ZMIN(IC2Z) < 1.0E-5) THEN
    HARR(18)=ZMAX(IC2Z)
  END IF
END IF
!
! RESET OTHER LABELS
IF (INSTAK > 0)  LABEL(INSTAK)='NSTACK'
IF (ICR > 0)  LABEL(ICR)='CONTACT_RES'
LABEL(IFLDUSEF)=LABUSEFLAG
CALL BDBHDR(LUHOUT,MODE,NFLD,LOCX,LOCY,NPTS,LSORT,LABEL,ZMIN,ZMAX,LTYPE)
CALL BDBXHDR(LUHOUT,MODE,HARR,HSTR)
!
! WIND UP
CLOSE(LUHIN)
CLOSE(LUHOUT)

RETURN

CONTAINS
  !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  SUBROUTINE RDESFHDR()
  !--------------------------------------------------------------------------------------
  !
  ! READS HEADER LINES OF ESF IP FILE
  !
  !---------------------------------------------------------------------------------------
  !
  CHARACTER(LEN=4000) :: ALINE
  CHARACTER(LEN=20), PARAMETER :: OUTSUB = 'RDESFHDR'
  CHARACTER(LEN=1), PARAMETER :: SPACE = CHAR(32)
  CHARACTER(LEN=1), PARAMETER :: TAB = CHAR(9)
  CHARACTER(LEN=1), PARAMETER :: COMMA =','
  CHARACTER(LEN=2), PARAMETER :: SPACECOMMA =' ,'
  CHARACTER(LEN=1), PARAMETER :: FORSLASH ='/'
  CHARACTER(LEN=10) :: DTYPESTR                   !DATA TYPE
  CHARACTER(LEN=10) :: UNITSTR                    !HOLDER FOR UNITS
  CHARACTER(LEN=2) :: DIRSTR
  CHARACTER(LEN=1000) :: TEMPSTR
  CHARACTER(LEN=30) :: KEYWORD
  CHARACTER(LEN=970) :: CONSTANT
  INTEGER :: IFLD
  INTEGER :: I
  INTEGER :: IATKPNTS                     !COUNTER FOR @KEYWORD ARRAY
  INTEGER :: ILEN
  INTEGER :: ICOL,IFLG,JCOL,JFLG
  INTEGER :: ISTART                       !STARTING FIELD FOR EXTRA DATA
  INTEGER :: IDELIMITER                   !CHARACTER COUNT FOR DELIMITER
  INTEGER :: IACTION
  INTEGER :: NSWIN,NEWIN                  !NUMBER OF START AND END TIMES SPECIFIED
  INTEGER :: IKEYWRD                     !KEYWORD COUNTER
  REAL :: TTOT
  REAL :: RTEMP
  REAL :: RINITDELAY                      !INIT DELAY IF USED
  REAL :: RLAT                            !HOLDER FOR LATITUDE FROM BOUNDARY KEYWORD
  REAL, DIMENSION(MAXTIMES-12) :: STIME   !STARTTIMES OR WIDTH -12 ALLOWS FOR ELECTRODE POSITIONS
  REAL, DIMENSION(MAXTIMES-12) :: ETIME   !END TIMES -12 ALLOWS FOR ELECTRODE POSITIONS
  REAL, DIMENSION(MAXTIMES/2-6) :: HARMF  !HARMONIC FREQUENCIES -6 IS HALF 12 AND  ALLOWS FOR ELECTRODE POSITIONS 
  LOGICAL :: LDONE                        !FLAG IF HEADER READ
  LOGICAL :: LFIRST                       !FLAG IF FIRST IP WINDOW FOUND
  LOGICAL :: LFIRSTLN                     !FLAG IF FIRST NON-BLANK LINE IN FILE IS VALID ESF
  LOGICAL :: LTOOMANYF                    !FLAG IF USER HAS ALREADY BEEN WARNED THAT THERE ARE TOO MANY FIELDS IN THE INPUT FILE
  LOGICAL :: LVNORM                       !FLAG IF Vp HAS BEEN CURRENT NORMALISED


  LDONE=.FALSE.
  LFIRST=.FALSE.
  LFIRSTLN=.FALSE.
  LTOOMANYF=.FALSE.
  LVNORM=.FALSE.
  IC1X=-1
  RINITDELAY=0.0
  NSWIN=0
  NEWIN=0
  ISTART=0
  DO WHILE (.NOT. LDONE)
    READ(LUHIN,'(A)',IOSTAT=IERR)ALINE
    SELECT CASE (IERR)
      CASE (:-1)
        OUTMESS='END OF FILE REACHED BEFORE A VALID HEADER FOUND - This is not a valid ASEG ESF &
        &file. If you can''t sort it out then send to Kim'
        CALL WERRORBOX(OUTMESS,OUTSUB)
      CASE (1:)
        WRITE(OUTMESS,'(A,I5,A,I3)')'PROBLEM READING DATA FILE HEADER, LINE ',JLINE+1,' FILE OPEN ON ',LUHIN
        CALL WERRORBOX(OUTMESS,OUTSUB)
      CASE (0)  ! NO ERROR - PROCESS LINE
        JLINE=JLINE+1
        !FIRST LINE WILL CONTAIN THE STRING 'VER:'
        !SECOND RECORD WILL HAVE = OR :
        !THIRD RECORD WILL HAVE @ AS WELL AS : OR =
        !FORTH RECORD WILL BE FIELD LABELS
        IF (ALINE(1:1) == '\') CYCLE
        IF (LEN_TRIM(ALINE) < 5) CYCLE
        CALL IUPPERCASE(ALINE)
        ! REPLACE ANY TABS WITH A SPACE
        IF (INDEX(ALINE,TAB) /= 0) THEN
          CALL TXTMAC('*',.TRUE.,.FALSE.,TAB,SPACE,ALINE,ILEN)          !S/R FROM CHARPAC
        END IF
        IF (INDEX(ALINE,'VER:')+INDEX(ALINE,'VER=') > 0) THEN
          LFIRSTLN=.TRUE.
          CYCLE
        END IF
        IF (.NOT. LFIRSTLN) THEN
          OUTMESS='First line of text in file does not contain VER: - This is not a valid ASEG ESF file'
          CALL WERRORBOX(OUTMESS,OUTSUB)
        END IF
        IF (INDEX(ALINE,':') + INDEX(ALINE,'=') > 0) THEN           !RECORD TYPE 2 OR 3
          ICOL=1
          IFLG=0
          IKEYWRD=0
          DO WHILE (IFLG == 0)
            IKEYWRD=IKEYWRD+1
            CALL TXTEXT(SPACE,ALINE,ICOL,TEMPSTR,IFLG)               !S/R FROM CHARPAC
            IF (IFLG > 0) THEN
              WRITE(OUTMESS,'("Problem parsing keyword ",I2," at line ",I2," from ",A)')IKEYWRD,JLINE,TRIM(ALINE)
              CALL WERRORBOX(OUTMESS,OUTSUB)
            END IF
            IDELIMITER=INDEX(TEMPSTR,':')+INDEX(TEMPSTR,'=')
            KEYWORD=TEMPSTR(1:IDELIMITER-1)
            CONSTANT=TEMPSTR(IDELIMITER+1:LEN_TRIM(TEMPSTR))
            SELECT CASE(TRIM(KEYWORD))
              CASE('ARRAY')
                IF (TRIM(CONSTANT).EQ.'DPDP') THEN
                  HSTR(6)=SURVEYTYPES(2)              !'DIPOLE-DIPOLE'
                ELSEIF (TRIM(CONSTANT).EQ.'GRD' .OR. TRIM(CONSTANT).EQ.'GRAD') THEN
                  HSTR(6)=SURVEYTYPES(1)              !'GRADIENT'
                ELSEIF (TRIM(CONSTANT).EQ.'PLDP' .OR. TRIM(CONSTANT).EQ.'PDP') THEN
                  HSTR(6)=SURVEYTYPES(3)              !'POLE-DIPOLE'
                ELSEIF (TRIM(CONSTANT).EQ.'DPPL') THEN
                  HSTR(6)=SURVEYTYPES(4)              !'DIPOLE-POLE'
                ELSEIF (TRIM(CONSTANT).EQ.'PLPL') THEN
                  HSTR(6)=SURVEYTYPES(5)              !'POLE-POLE'
                ELSEIF (TRIM(CONSTANT).EQ.'SCHL') THEN
                  HSTR(6)=SURVEYTYPES(6)              !'SCHLUMBERGER'
                ELSEIF (TRIM(CONSTANT).EQ.'WENN') THEN
                  HSTR(6)=SURVEYTYPES(7)              !'WENNER'
                ELSEIF (TRIM(CONSTANT).EQ.'USER') THEN
                  HSTR(6)=SURVEYTYPES(12)             !'MIXED_ARRAY'
                ELSE
                  WRITE (OUTMESS,'("THE ",A," ARRAY HAS NOT BEEN PROCESSED BEFORE - IT WILL BE TREATED AS A MIXED ARRAY")')TRIM(CONSTANT)
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  HSTR(6)=SURVEYTYPES(12)             !'MIXED_ARRAY'
                END IF
              CASE('AZIMUTH')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("AZUMITH: ",A)')TRIM(CONSTANT)
              CASE('@BOUNDARY')
                IATKPNTS=0  
                JCOL=1
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("BOUNDARY DETAILS FOUND",/,"LATITUDE LONGITUDE")')
                DO 
                  CALL SExtNum(JCOL,CONSTANT,COMMA,RTEMP,JFLG)               !S/R FROM CHARPAC
                  IF (JFLG > 0) THEN
                    WRITE(OUTMESS,'("Problem boundary point ",I2," from ",A)')IATKPNTS+1,TRIM(CONSTANT)
                    CALL WERRORBOX(OUTMESS,OUTSUB)
                  END IF
                  IATKPNTS=IATKPNTS+1   
                  IF (MOD(IATKPNTS,2) == 0) THEN
                    WRITE(LUHPRT,'(2G13.6)')RLAT,RTEMP
                  ELSE
                    RLAT=RTEMP
                  END IF
                  IF (JFLG < 0) EXIT
                END DO        
              CASE('BOUNDFILE')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("BOUNDARY FILE NAME: ",A)')TRIM(CONSTANT)
              CASE('C1Y')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(14)=RTEMP
              CASE('F1Y')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(14)=RTEMP
              CASE('FY1')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(14)=RTEMP
              CASE('T1Y')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(14)=RTEMP
              CASE('TY1')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(14)=RTEMP
              CASE('C1N')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(14)=RTEMP
              CASE('C1NORTH')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(14)=RTEMP
              CASE('C1_LINE')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(14)=RTEMP
              CASE('C1Z')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(15)=RTEMP
              CASE('C1D')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(15)=RTEMP
              CASE('T1Z')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(15)=RTEMP
              CASE('C1LEVEL')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(15)=RTEMP
              CASE('C2X')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(16)=RTEMP
              CASE('F2X')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(16)=RTEMP
              CASE('FX2')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(16)=RTEMP
              CASE('T2X')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(16)=RTEMP
              CASE('TX2')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(16)=RTEMP
              CASE('C2E')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(16)=RTEMP
              CASE('C2EAST')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(16)=RTEMP
              CASE('C2_STATION')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(16)=RTEMP
              CASE('C2Y')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(17)=RTEMP
              CASE('F2Y')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(17)=RTEMP
              CASE('FY2')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(17)=RTEMP
              CASE('T2Y')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(17)=RTEMP
              CASE('TY2')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(17)=RTEMP
              CASE('C2N')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(17)=RTEMP
              CASE('C2NORTH')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(17)=RTEMP
              CASE('C2_LINE')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(17)=RTEMP
              CASE('C2D')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(18)=RTEMP
              CASE('M2Z')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(18)=RTEMP
              CASE('R2Z')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(18)=RTEMP
              CASE('C2LEVEL')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(18)=RTEMP
              CASE('CHANNELFILE')
                CHFNAME=TRIM(CONSTANT)
              CASE('COLLARFILE')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("COLLAR FILE DECLARED: ",A)')TRIM(CONSTANT)
              CASE('CONTRACTOR')
                HSTR(4)=TRIM(CONSTANT)
              CASE('CURRENT')  
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                IF(LVNORM) THEN
                  WRITE(OUTMESS,'("The primary voltage has been current normalised but a fixed &
                    &value for the current of ",G12.5," has been found in the header. The current &
                    &in the database will be set to 1.0A to allow for apparent resistivity calculation &
                    &but the value supplied in the header will be reported to the PRT file.")')RTEMP
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  !REPORT TO THE PRT FILE FOR THE MOMENT
                  WRITE(LUHPRT,'("FIXED CURRENT SET TO: ",G12.5)')RTEMP
                ELSE
                  RCURR=RTEMP
                END IF 
              CASE('DATATYPE')
                IF (TRIM(CONSTANT) /= 'DCRES' .AND. TRIM(CONSTANT) /= 'TDIP' .AND. &
                  &TRIM(CONSTANT) /= 'FDIP') THEN
                  WRITE(OUTMESS,'(" The datatype has been set to ",A," which is not a type that IPProc &
                    &expects. Do you want to continue parsing or use a more approprate input program? &
                    &Continue?")')TRIM(CONSTANT)
                  CALL WYESNOBOX(OUTMESS,OUTSUB,IACTION)
                  IF (IACTION == 0) STOP
                  DTYPESTR=TRIM(CONSTANT)
                ELSEIF(TRIM(CONSTANT) == 'TDIP') THEN
                  LTDOMAIN=.TRUE.
                END IF
              CASE('DATE')
                HSTR(5)=TRIM(CONSTANT)
              CASE('DATUM')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("DATUM: ",A)')TRIM(CONSTANT)
              CASE('DIPOLE')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(12)=RTEMP
              CASE('@DELAY')
                !GET WINDOW TIMES
                JCOL=1
                I=1
                JFLG=0
                DO WHILE (JFLG == 0)
                  CALL SExtNum(JCOL,CONSTANT,COMMA,WTIME(I),JFLG)               !S/R FROM CHARPAC
                  IF (JFLG > 0) EXIT
                  I=I+1
                  IF (I > MAXTIMES-12 .AND. JFLG == 0) THEN         !-12 allows room for electrodes
                    WRITE(OUTMESS,'("Entertaining but possibly not very clever! This file has more than ",I3," windows &
                      &which is the maximum the database can handle. Only the first ",I3," will be loaded")')MAXTIMES-12,MAXTIMES-12
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LTOOMANYF=.TRUE.
                    EXIT
                  END IF
                END DO
                NWIN=I-1
                IF (NWINH > 0. .AND. NWIN /= NWINH) THEN
                  WRITE (OUTMESS,'("The header says there are ",I3," time windows in the input file but there &
                    &are ",I3," window times. The lesser of the two will take precedence")')NWINH,NWIN
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  NWIN=MIN(NWIN,NWINH)
                END IF
                ! NOW WE KNOW HOW MANY WINDOWS THERE ARE, SET THE OUTPUT FIELDS FOR THE ELECTRODES ETC.
                ISTART=NWIN+10+1
                IC1X=ISTART
                IC1Y=ISTART+1
                IC1Z=ISTART+2
                IC2X=ISTART+3
                IC2Y=ISTART+4
                IC2Z=ISTART+5
                IP1X=ISTART+6
                IP1Y=ISTART+7
                IP1Z=ISTART+8
                IP2X=ISTART+9
                IP2Y=ISTART+10
                IP2Z=ISTART+11
                IEND=ISTART+11
                ISTDDEV=0
                ISP=0
                INSTAK=0
                ICR=0
                LSPECTRAL=.TRUE.
              CASE('TIMES')
                !GET WINDOW TIMES
                JCOL=1
                I=1
                JFLG=0
                DO WHILE (JFLG == 0)
                  CALL SExtNum(JCOL,CONSTANT,COMMA,WTIME(I),JFLG)
                  IF (JFLG > 0) EXIT
                  I=I+1
                  IF (I > MAXTIMES-12 .AND. JFLG == 0) THEN         !-12 allows room for electrodes
                    WRITE(OUTMESS,'("Entertaining but possibly not very clever! This file has more than ",I3," windows &
                      &which is the maximum the database can handle. Only the first ",I3," will be loaded")')MAXTIMES-12,MAXTIMES-12
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LTOOMANYF=.TRUE.
                    EXIT
                  END IF
                END DO
                NWIN=I-1
                IF (NWINH > 0. .AND. NWIN /= NWINH) THEN
                  WRITE (OUTMESS,'("The header says there are ",I3," time windows in the input file but there &
                    &are ",I3," window times. The lesser of the two will take precedence")')NWINH,NWIN
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  NWIN=MIN(NWIN,NWINH)
                END IF
                ! NOW WE KNOW HOW MANY WINDOWS THERE ARE SET THE OUTPUT FIELDS FOR THE ELECTRODES ETC.
                ISTART=NWIN+10+1
                IC1X=ISTART
                IC1Y=ISTART+1
                IC1Z=ISTART+2
                IC2X=ISTART+3
                IC2Y=ISTART+4
                IC2Z=ISTART+5
                IP1X=ISTART+6
                IP1Y=ISTART+7
                IP1Z=ISTART+8
                IP2X=ISTART+9
                IP2Y=ISTART+10
                IP2Z=ISTART+11
                IEND=ISTART+11
                ISTDDEV=0
                ISP=0
                INSTAK=0
                ICR=0
                LSPECTRAL=.TRUE.
              CASE('DUTYCYCLE')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("DUTY CYLCE: ",A)')TRIM(CONSTANT)
              CASE('DC')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("DUTY CYLCE: ",A)')TRIM(CONSTANT)
              CASE('@F')
                !GET NUMBER OF FREQUENCIES AND THEIR VALUES
                JCOL=1
                I=1
                JFLG=0
                DO WHILE (JFLG == 0)
                  CALL SExtNum(JCOL,CONSTANT,COMMA,HARMF(I),JFLG)
                  IF (JFLG > 0) EXIT
                  I=I+1
                  IF (I > (MAXTIMES-12)/2 .AND. JFLG == 0) THEN
                    WRITE(OUTMESS,'("Entertaining but possibly not very clever! This file has more than ",I3, &
                      &" harmonics which is the maximum the database can handle. Only the first ",I3, &
                      &" will be loaded")')(MAXTIMES-12)/2,(MAXTIMES-12)/2
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LTOOMANYF=.TRUE.
                    EXIT
                  END IF
                END DO
                NHARM=I-1
                HARR(4)=HARMF(1)
                IF (NWIN > 0) NWIN=0
                ! NOW WE KNOW HOW MANY HARMONICS THERE ARE SET THE OUTPUT FIELDS FOR THE ELECTRODES ETC.
                ISTART=NHARM*2+10+1
                IC1X=ISTART
                IC1Y=ISTART+1
                IC1Z=ISTART+2
                IC2X=ISTART+3
                IC2Y=ISTART+4
                IC2Z=ISTART+5
                IP1X=ISTART+6
                IP1Y=ISTART+7
                IP1Z=ISTART+8
                IP2X=ISTART+9
                IP2Y=ISTART+10
                IP2Z=ISTART+11
                IEND=ISTART+11
                ISTDDEV=0
                ISP=0
                INSTAK=0
                ICR=0
                LSPECTRAL=.TRUE.
              CASE('INITDELAY')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("INITIAL DELAY: ",A)')TRIM(CONSTANT)
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                RINITDELAY=RTEMP                
              CASE('INIT')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("INITIAL DELAY: ",A)')TRIM(CONSTANT)
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                RINITDELAY=RTEMP                
              CASE('INIT_DELAY')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("INITIAL DELAY: ",A)')TRIM(CONSTANT)
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                RINITDELAY=RTEMP                
              CASE('INSTRUMENT')
                IF (LEN_TRIM(HSTR(8)) > 0) THEN           !ALREADY SET BY RECEIVER KEYWORD
                  WRITE(LUHPRT,'("INSTRUMENT: ",A)')TRIM(CONSTANT)
                ELSE  
                  HSTR(8)=TRIM(CONSTANT)
                END IF
              CASE('INST')
                IF (LEN_TRIM(HSTR(8)) > 0) THEN           !ALREADY SET BY RECEIVER KEYWORD
                  WRITE(LUHPRT,'("INSTRUMENT: ",A)')TRIM(CONSTANT)
                ELSE  
                  HSTR(8)=TRIM(CONSTANT)
                END IF
              CASE('LINE')
                CALL GETDIRECTION(CONSTANT,RLINE,DIRSTR)
                HSTR(10)=DIRSTR
              CASE('BOREHOLE')
                CALL GETDIRECTION(CONSTANT,RLINE,DIRSTR)
                HSTR(10)='DH'
              CASE('PROFILE')
                CALL GETDIRECTION(CONSTANT,RLINE,DIRSTR)
                HSTR(10)=DIRSTR
              CASE('MX_START')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(5)=RTEMP
              CASE('MX_END')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(6)=RTEMP
              CASE('NORTHTYPE')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("NORTH TYPE: ",A)')TRIM(CONSTANT)
              CASE('NORMV')
                IF (TRIM(CONSTANT) == 'YES') THEN
                  !REPORT TO THE PRT FILE FOR THE MOMENT
                  WRITE(LUHPRT,'("THE PRIMARY VOLTAGE HAS BEEN CURRENT NORMALISED")')
                  IF(.NOT. LNULL(RCURR) .AND. (ABS(RCURR)-1.0) > 1.0E-3) THEN
                    WRITE(OUTMESS,'("The primary voltage has been current normalised but a fixed &
                      &value for the current of ",G12.5," has been found in the header. The current &
                      &in the database will be set to 1.0A to allow for apparent resistivity calculation &
                      &but the value supplied in the header will be reported to the PRT file.")')RCURR
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    WRITE(LUHPRT,'("FIXED CURRENT SET TO: ",G12.5)')RCURR
                    LVNORM=.TRUE.  
                  END IF 
                  RCURR=1.0
                END IF
              CASE('NULL')
                NULLSTR=TRIM(CONSTANT)
              CASE('NUMTIMES')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                NWINH=INT(RTEMP)
                IF (NWIN > 0. .AND. NWIN /= NWINH) THEN
                  WRITE (OUTMESS,'("The header says there are ",I3," time windows in the input file but there &
                    &are ",I3," window times. The number of database fields has already been set according to &
                    &the number of times. If this is a problem then move the NUMTIMES keyword before the T= &
                    &keyword in the ESF file")')NWINH,NWIN
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                END IF
              CASE('NCH')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                NWINH=INT(RTEMP)
                IF (NWIN > 0. .AND. NWIN /= NWINH) THEN
                  WRITE (OUTMESS,'("The header says there are ",I3," time windows in the input file but there &
                    &are ",I3," window times. The number of database fields has already been set according to &
                    &the number of times. If this is a problem then move the NUMTIMES keyword before the T= &
                    &keyword in the ESF file")')NWINH,NWIN
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                END IF
              CASE('WINDOWS')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                NWINH=INT(RTEMP)
                IF (NWIN > 0. .AND. NWIN /= NWINH) THEN
                  WRITE (OUTMESS,'("The header says there are ",I3," time windows in the input file but there &
                    &are ",I3," window times. The number of database fields has already been set according to &
                    &the number of times. If this is a problem then move the NUMTIMES keyword before the T= &
                    &keyword in the ESF file")')NWINH,NWIN
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                END IF
              CASE('OFFTIME')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                OFFTIME=RTEMP
              CASE('ONTIME')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                ONTIME=RTEMP
              CASE('PULSEWIDTH')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("ON TIME: ",A)')TRIM(CONSTANT)
              CASE('P1Y')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P1Y=RTEMP
              CASE('R1Y')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P1Y=RTEMP
              CASE('M1Y')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P1Y=RTEMP
              CASE('MY1')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P1Y=RTEMP
              CASE('P1_LINE')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P1Y=RTEMP
              CASE('P1Z')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P1Z=RTEMP
              CASE('P1D')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P1Z=RTEMP
              CASE('P2X')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P2X=RTEMP
              CASE('M2X')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P2X=RTEMP
              CASE('R2X')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P2X=RTEMP
              CASE('P2')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P2X=RTEMP
              CASE('M2')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P2X=RTEMP
              CASE('P2Y')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P2Y=RTEMP
              CASE('M2Y')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P2Y=RTEMP
              CASE('R2Y')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P2Y=RTEMP
              CASE('MY2')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P2Y=RTEMP
              CASE('P2_LINE')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P2Y=RTEMP
              CASE('P2Z')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P2Z=RTEMP
              CASE('P2D')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                P2Z=RTEMP
              CASE('PROJECT')
                HSTR(3)=TRIM(CONSTANT)
              CASE('PROJFILE')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("PROJECTION FILE DECLARED: ",A)')TRIM(CONSTANT)
              CASE('RECEIVER')
                IF (LEN_TRIM(HSTR(8)) > 0) THEN             !MUST HAVE BEEN LOADED BY INSTRUMENT KEYWORD
                  WRITE(LUHPRT,'("INSTRUMENT: ",A)')TRIM(HSTR(8))
                END IF   
                HSTR(8)=TRIM(CONSTANT)
              CASE('RX')
                IF (LEN_TRIM(HSTR(8)) > 0) THEN             !MUST HAVE BEEN LOADED BY INSTRUMENT KEYWORD
                  WRITE(LUHPRT,'("INSTRUMENT: ",A)')TRIM(HSTR(8))
                END IF   
                HSTR(8)=TRIM(CONSTANT)
              CASE('SENSOR')
                HSTR(9)=TRIM(CONSTANT)
              CASE('SRVFILE')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("SURVEY FILE DECLARED: ",A)')TRIM(CONSTANT)
              CASE('@T')
                !GET WINDOW TIMES
                JCOL=1
                I=1
                JFLG=0  
                DO WHILE (JFLG == 0)
                  CALL SExtNum(JCOL,CONSTANT,COMMA,STIME(I),JFLG)
                  IF (JFLG > 0) EXIT
                  I=I+1
                  IF (I > MAXTIMES-12) THEN
                    WRITE(OUTMESS,'("Entertaining but possibly not very clever! This file has more than ",I3," windows &
                      &which is the maximum the database can handle. Only the first ",I3," will be loaded")')MAXTIMES-12,MAXTIMES-12
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LTOOMANYF=.TRUE.
                    EXIT
                  END IF
                END DO
                NWIN=I-2
                TTOT=STIME(1)
                DO I=1,NWIN           !COMPUTE CENTRE TIME FOR EACH WINDOW
                  WTIME(I)=TTOT+STIME(I+1)/2
                  TTOT=TTOT+STIME(I+1)
                END DO
                IF (NWINH > 0. .AND. NWIN /= NWINH) THEN
                  WRITE (OUTMESS,'("The header says there are ",I3," time windows in the input file but there &
                    &are ",I3," window times. The lesser of the two will take precedence")')NWINH,NWIN
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  NWIN=MIN(NWIN,NWINH)
                END IF
                ! NOW WE KNOW HOW MANY WINDOWS THERE ARE SET THE OUTPUT FIELDS FOR THE ELECTRODES ETC.
                ISTART=NWIN+10+1
                IC1X=ISTART
                IC1Y=ISTART+1
                IC1Z=ISTART+2
                IC2X=ISTART+3
                IC2Y=ISTART+4
                IC2Z=ISTART+5
                IP1X=ISTART+6
                IP1Y=ISTART+7
                IP1Z=ISTART+8
                IP2X=ISTART+9
                IP2Y=ISTART+10
                IP2Z=ISTART+11
                IEND=ISTART+11
                ISTDDEV=0
                ISP=0
                INSTAK=0
                ICR=0
                LSPECTRAL=.TRUE.
              CASE('@TIMESSTART')
                !GET WINDOW START TIMES
                JCOL=1
                I=1
                JFLG=0  
                DO WHILE (JFLG == 0)
                  CALL SExtNum(JCOL,CONSTANT,COMMA,STIME(I),JFLG)
                  IF (JFLG > 0) EXIT
                  I=I+1
                  IF (I > MAXTIMES-12 .AND. JFLG == 0) THEN
                    WRITE(OUTMESS,'("Entertaining but possibly not very clever! This file has more than ",I3," windows &
                      &which is the maximum the database can handle. Only the first ",I3," will be loaded")')MAXTIMES-12,MAXTIMES-12
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LTOOMANYF=.TRUE.
                    EXIT
                  END IF
                END DO
                NSWIN=I-1
                IF (NEWIN > 0) THEN
                  NWIN=NEWIN  
                  IF (NEWIN /= NSWIN) THEN
                    WRITE (OUTMESS,'("The header says there are ",I3," window start times but ",I3," window end times! &
                      & The lesser of the two will take precedence")')NSWIN,NEWIN
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    NWIN=MIN(NSWIN,NEWIN)  
                  END IF          
                  IF (NWINH > 0. .AND. NSWIN /= NWINH) THEN
                    WRITE (OUTMESS,'("The header says there are ",I3," time windows in the input file but there &
                      &are ",I3," window times. The lesser of the two will take precedence")')NWINH,NSWIN
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    NWIN=MIN(NSWIN,NWINH)
                  END IF
                  DO I=1,NWIN           !COMPUTE CENTRE TIME FOR EACH WINDOW
                    WTIME(I)=(STIME(I)+ETIME(I))/2.0
                  END DO
                  ! NOW WE KNOW HOW MANY WINDOWS THERE ARE SET THE OUTPUT FIELDS FOR THE ELECTRODES ETC.
                  ISTART=NWIN+10+1
                  IC1X=ISTART
                  IC1Y=ISTART+1
                  IC1Z=ISTART+2
                  IC2X=ISTART+3
                  IC2Y=ISTART+4
                  IC2Z=ISTART+5
                  IP1X=ISTART+6
                  IP1Y=ISTART+7
                  IP1Z=ISTART+8
                  IP2X=ISTART+9
                  IP2Y=ISTART+10
                  IP2Z=ISTART+11
                  IEND=ISTART+11
                  ISTDDEV=0
                  ISP=0
                  INSTAK=0
                  ICR=0
                  LSPECTRAL=.TRUE.
                END IF
              CASE('@TIMESEND')
                !GET WINDOW END TIMES
                JCOL=1
                I=1
                JFLG=0  
                DO WHILE (JFLG == 0)
                  CALL SExtNum(JCOL,CONSTANT,COMMA,ETIME(I),JFLG)
                  IF (JFLG > 0) EXIT
                  I=I+1
                  IF (I > MAXTIMES-12 .AND. JFLG == 0) THEN
                    WRITE(OUTMESS,'("Entertaining but possibly not very clever! This file has more than ",I3," windows &
                      &which is the maximum the database can handle. Only the first ",I3," will be loaded")')MAXTIMES-12,MAXTIMES-12
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LTOOMANYF=.TRUE.
                    EXIT
                  END IF
                END DO
                NEWIN=I-1
                IF (NSWIN > 0) THEN
                  NWIN=NEWIN  
                  IF (NEWIN /= NSWIN) THEN
                    WRITE (OUTMESS,'("The header says there are ",I3," window start times but ",I3," window end times! &
                      & The lesser of the two will take precedence")')NSWIN,NEWIN
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    NWIN=MIN(NSWIN,NEWIN)  
                  END IF          
                  IF (NWINH > 0. .AND. NSWIN /= NWINH) THEN
                    WRITE (OUTMESS,'("The header says there are ",I3," time windows in the input file but there &
                      &are ",I3," window times. The lesser of the two will take precedence")')NWINH,NSWIN
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    NWIN=MIN(NSWIN,NWINH)
                  END IF
                  DO I=1,NWIN           !COMPUTE CENTRE TIME FOR EACH WINDOW
                    WTIME(I)=(STIME(I)+ETIME(I))/2.0
                  END DO
                  ! NOW WE KNOW HOW MANY WINDOWS THERE ARE SET THE OUTPUT FIELDS FOR THE ELECTRODES ETC.
                  ISTART=NWIN+10+1
                  IC1X=ISTART
                  IC1Y=ISTART+1
                  IC1Z=ISTART+2
                  IC2X=ISTART+3
                  IC2Y=ISTART+4
                  IC2Z=ISTART+5
                  IP1X=ISTART+6
                  IP1Y=ISTART+7
                  IP1Z=ISTART+8
                  IP2X=ISTART+9
                  IP2Y=ISTART+10
                  IP2Z=ISTART+11
                  IEND=ISTART+11
                  ISTDDEV=0
                  ISP=0
                  INSTAK=0
                  ICR=0
                  LSPECTRAL=.TRUE.
                END IF
              CASE('TRANSMITTER')
                HSTR(7)=TRIM(CONSTANT)
              CASE('TRANS')
                HSTR(7)=TRIM(CONSTANT)
              CASE('TX')
                HSTR(7)=TRIM(CONSTANT)
              CASE('TXFREQ')
                CALL CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
                HARR(4)=RTEMP
              CASE('UNITS.EMIP')
                DO I=1,NIPUNITS
                  UNITSTR=UNITSIP(I)
                  CALL IUPPERCASE(UNITSTR)
                  IF (TRIM(CONSTANT) == TRIM(UNITSTR)) THEN
                    HSTR(13)=UNITSIP(I)
                    EXIT
                  END IF
                END DO
                IF (I > NIPUNITS) HSTR(13)=TRIM(CONSTANT)
              CASE('UNITS.LENGTH')
                IF (TRIM(CONSTANT) =='M') THEN
                  HSTR(11)='Metres'
                ELSEIF (TRIM(CONSTANT) =='FT') THEN
                  HSTR(11)='Feet'
                ELSE
                  HSTR(11)=TRIM(CONSTANT)
                END IF
              CASE('LENGTH_UNITS')
                IF (TRIM(CONSTANT) =='M') THEN
                  HSTR(11)='Metres'
                ELSEIF (TRIM(CONSTANT) =='FT') THEN
                  HSTR(11)='Feet'
                ELSE
                  HSTR(11)=TRIM(CONSTANT)
                END IF
              CASE('UNITS')
                IF (TRIM(CONSTANT) =='M') THEN
                  HSTR(11)='Metres'
                ELSEIF (TRIM(CONSTANT) =='FT') THEN
                  HSTR(11)='Feet'
                ELSE
                  HSTR(11)=TRIM(CONSTANT)
                END IF
              CASE('@WIDTH')
                !GET WINDOW WIDTH TIMES IF CENTRE TIMES NOT ALREADY GIVEN
                IF (NWIN > 0) CYCLE
                JCOL=1
                I=1
                JFLG=0  
                DO WHILE (JFLG == 0)
                  CALL SExtNum(JCOL,CONSTANT,COMMA,STIME(I),JFLG)
                  IF (JFLG > 0) EXIT
                  I=I+1
                  IF (I > MAXTIMES-12 .AND. JFLG == 0) THEN
                    WRITE(OUTMESS,'("Entertaining but possibly not very clever! This file has more than ",I3," windows &
                      &which is the maximum the database can handle. Only the first ",I3," will be loaded")')MAXTIMES-12,MAXTIMES-12
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LTOOMANYF=.TRUE.
                    EXIT
                  END IF
                END DO
                NWIN=I-1
                IF (NWINH > 0. .AND. NWIN /= NWINH) THEN
                  WRITE (OUTMESS,'("The header says there are ",I3," time windows in the input file but there &
                    &are ",I3," window times. The lesser of the two will take precedence")')NWINH,NWIN
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  NWIN=MIN(NWIN,NWINH)
                END IF
                TTOT=RINITDELAY
                DO I=1,NWIN           !COMPUTE CENTRE TIME FOR EACH WINDOW
                  WTIME(I)=TTOT+STIME(I)/2
                  TTOT=TTOT+STIME(I)
                END DO
                ! NOW WE KNOW HOW MANY WINDOWS THERE ARE SET THE OUTPUT FIELDS FOR THE ELECTRODES ETC.
                ISTART=NWIN+10+1
                IC1X=ISTART
                IC1Y=ISTART+1
                IC1Z=ISTART+2
                IC2X=ISTART+3
                IC2Y=ISTART+4
                IC2Z=ISTART+5
                IP1X=ISTART+6
                IP1Y=ISTART+7
                IP1Z=ISTART+8
                IP2X=ISTART+9
                IP2Y=ISTART+10
                IP2Z=ISTART+11
                IEND=ISTART+11
                ISTDDEV=0
                ISP=0
                INSTAK=0
                ICR=0
                LSPECTRAL=.TRUE.
              CASE('WIREFILE')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("TX WIRE PATH FILE DECLARED: ",A)')TRIM(CONSTANT)
              CASE('XCOLLAR')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("DRILLHOLE COLLAR X: ",A)')TRIM(CONSTANT)
              CASE('YCOLLAR')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("DRILLHOLE COLLAR Y: ",A)')TRIM(CONSTANT)
              CASE('ZCOLLAR')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("DRILLHOLE COLLAR Z: ",A)')TRIM(CONSTANT)
              CASE('ZONE')
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'("UTM ZONE: ",A)')TRIM(CONSTANT)
              CASE DEFAULT
                !REPORT TO THE PRT FILE FOR THE MOMENT
                WRITE(LUHPRT,'(A,": ",A)')TRIM(KEYWORD),TRIM(CONSTANT)
            END SELECT  
          END DO
        ELSE          !MUST BE A LABEL LINE
          IF (ISTART == 0 .AND. LEN_TRIM(CHFNAME) > 0) THEN
            CALL RWCHANFL(CHFNAME,NWIN,WTIME,OFFTIME,MODE)
            ISTART=NWIN+10+1
            IC1X=ISTART
            IC1Y=ISTART+1
            IC1Z=ISTART+2
            IC2X=ISTART+3
            IC2Y=ISTART+4
            IC2Z=ISTART+5
            IP1X=ISTART+6
            IP1Y=ISTART+7
            IP1Z=ISTART+8
            IP2X=ISTART+9
            IP2Y=ISTART+10
            IP2Z=ISTART+11
            IEND=ISTART+11
            ISTDDEV=0
            ISP=0
            INSTAK=0
            ICR=0
            LSPECTRAL=.TRUE.
          END IF
          LDONE=.TRUE.
          ICOL=1
          IFLG=0
          IFLD=0
          IKEYWRD=0
          DO WHILE (IFLG == 0)
            IKEYWRD=IKEYWRD+1
            CALL TXTEXT(SPACE,ALINE,ICOL,KEYWORD,IFLG)
            IF (IFLG > 0) THEN
              WRITE(OUTMESS,'("Problem parsing keyword ",I3," at line ",I2," from ",A)')IKEYWRD,JLINE,TRIM(ALINE)
              CALL WERRORBOX(OUTMESS,OUTSUB)
            END IF
            IFLD=IFLD+1
            SELECT CASE(TRIM(KEYWORD))
              CASE('C1X')
                !CURRENT ELECTRODE 1 X VALUE
                JACOL(IFLD)=IC1X
              CASE('F1X')
                !CURRENT ELECTRODE 1 X VALUE
                JACOL(IFLD)=IC1X
              CASE('C1')
                !CURRENT ELECTRODE 1 X VALUE
                JACOL(IFLD)=IC1X
              CASE('FX1')
                !CURRENT ELECTRODE 1 X VALUE
                JACOL(IFLD)=IC1X
              CASE('T1X')
                !CURRENT ELECTRODE 1 X VALUE
                JACOL(IFLD)=IC1X
              CASE('TX1')
                !CURRENT ELECTRODE 1 X VALUE
                JACOL(IFLD)=IC1X
              CASE('C1E')
                !CURRENT ELECTRODE 1 X VALUE
                JACOL(IFLD)=IC1X
              CASE('C1EAST')
                !CURRENT ELECTRODE 1 X VALUE
                JACOL(IFLD)=IC1X
              CASE('C1_STATION')
                !CURRENT ELECTRODE 1 X VALUE
                JACOL(IFLD)=IC1X
              CASE('C1Y')
                !CURRENT ELECTRODE 1 Y VALUE
                JACOL(IFLD)=IC1Y
              CASE('F1Y')
                !CURRENT ELECTRODE 1 Y VALUE
                JACOL(IFLD)=IC1Y
              CASE('FY1')
                !CURRENT ELECTRODE 1 Y VALUE
                JACOL(IFLD)=IC1Y
              CASE('T1Y')
                !CURRENT ELECTRODE 1 Y VALUE
                JACOL(IFLD)=IC1Y
              CASE('TY1')
                !CURRENT ELECTRODE 1 Y VALUE
                JACOL(IFLD)=IC1Y
              CASE('C1N')
                !CURRENT ELECTRODE 1 Y VALUE
                JACOL(IFLD)=IC1Y
              CASE('C1NORTH')
                !CURRENT ELECTRODE 1 Y VALUE
                JACOL(IFLD)=IC1Y
              CASE('C1_LINE')
                !CURRENT ELECTRODE 1 Y VALUE
                JACOL(IFLD)=IC1Y
              CASE('C1Z')
                !CURRENT ELECTRODE 1 Z VALUE
                JACOL(IFLD)=IC1Z
              CASE('C1D')
                !CURRENT ELECTRODE 1 Z VALUE
                JACOL(IFLD)=IC1Z
              CASE('R1Z')
                !CURRENT ELECTRODE 1 Z VALUE
                JACOL(IFLD)=IC1Z
              CASE('C1LEVEL')
                !CURRENT ELECTRODE 1 Z VALUE
                JACOL(IFLD)=IC1Z
              CASE('C2X')
                !CURRENT ELECTRODE 2 X VALUE
                JACOL(IFLD)=IC2X
              CASE('F2X')
                !CURRENT ELECTRODE 2 X VALUE
                JACOL(IFLD)=IC2X
              CASE('C2')
                !CURRENT ELECTRODE 2 X VALUE
                JACOL(IFLD)=IC2X
              CASE('F2')
                !CURRENT ELECTRODE 2 X VALUE
                JACOL(IFLD)=IC2X
              CASE('FX2')
                !CURRENT ELECTRODE 2 X VALUE
                JACOL(IFLD)=IC2X
              CASE('T2X')
                !CURRENT ELECTRODE 2 X VALUE
                JACOL(IFLD)=IC2X
              CASE('TX2')
                !CURRENT ELECTRODE 2 X VALUE
                JACOL(IFLD)=IC2X
              CASE('C2E')
                !CURRENT ELECTRODE 2 X VALUE
                JACOL(IFLD)=IC2X
              CASE('C2EAST')
                !CURRENT ELECTRODE 2 X VALUE
                JACOL(IFLD)=IC2X
              CASE('C2_STATION')
                !CURRENT ELECTRODE 2 X VALUE
                JACOL(IFLD)=IC2X
              CASE('C2Y')
                !CURRENT ELECTRODE 2 Y VALUE
                JACOL(IFLD)=IC2Y
              CASE('F2Y')
                !CURRENT ELECTRODE 2 Y VALUE
                JACOL(IFLD)=IC2Y
              CASE('T2Y')
                !CURRENT ELECTRODE 2 Y VALUE
                JACOL(IFLD)=IC2Y
              CASE('FY2')
                !CURRENT ELECTRODE 2 Y VALUE
                JACOL(IFLD)=IC2Y
              CASE('TY2')
                !CURRENT ELECTRODE 2 Y VALUE
                JACOL(IFLD)=IC2Y
              CASE('C2N')
                !CURRENT ELECTRODE 2 Y VALUE
                JACOL(IFLD)=IC2Y
              CASE('C2NORTH')
                !CURRENT ELECTRODE 2 Y VALUE
                JACOL(IFLD)=IC2Y
              CASE('C2_LINE')
                !CURRENT ELECTRODE 2 Y VALUE
                JACOL(IFLD)=IC2Y
              CASE('C2Z')
                !CURRENT ELECTRODE 2 Z VALUE
                JACOL(IFLD)=IC2Z
              CASE('C2D')
                !CURRENT ELECTRODE 2 Z VALUE
                JACOL(IFLD)=IC2Z
              CASE('F2Z')
                !CURRENT ELECTRODE 2 Z VALUE
                JACOL(IFLD)=IC2Z
              CASE('T2Z')
                !CURRENT ELECTRODE 2 Z VALUE
                JACOL(IFLD)=IC2Z
              CASE('C2LEVEL')
                !CURRENT ELECTRODE 2 Z VALUE
                JACOL(IFLD)=IC2Z
              CASE('CH0':'CH999')
                IF (.NOT. LFIRST .AND. NWIN > 0 ) THEN
                  !FIRST CHARGEABILITY WINDOW
                  LFIRST=.TRUE.
                  JACOL(IFLD)=IIP1
                  DO I=1,NWIN-1
                    JACOL(IFLD+I)=IIP1+I
                  END DO
                ELSEIF (NWIN == 0) THEN
                  IF (IFLDEXTRA(MAXEXTRAF) > 0) THEN
                    WRITE(OUTMESS,'("This data set has over ",i2," field labels which have not been encountered &
                    &before! Very creative but irritating! The field labelled ",A," has not been encountered before &
                    &and it and any other new fields after it will be ignored. If any of these fields are required see Kim")') &
                    &MAXEXTRAF,TRIM(TEMPSTR)
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LWARNLABEL=.FALSE.
                  ELSEIF (IEND < MAXFLDS) THEN
                    IEND=IEND+1
                    I=1
                    DO WHILE (IFLDEXTRA(I) >0)
                     I=I+1
                    END DO
                    IFLDEXTRA(I)=IFLD
                    LABEXTRA(I)=TRIM(KEYWORD)
                    JACOL(IFLD)=4000+I                !PUSH OUT OF DATABASE FIELD RANGE UNTIL WE KNOW HOW MANY WE HAVE
                  ELSEIF (.NOT. LTOOMANYF) THEN
                    WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                      &Extra fields will be ignored")')MAXFLDS
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LTOOMANYF=.TRUE.
                  END IF
                END IF
              CASE('IP0':'IP999')
                IF (.NOT. LFIRST .AND. NWIN > 0) THEN
                  !FIRST CHARGEABILITY WINDOW
                  LFIRST=.TRUE.
                  JACOL(IFLD)=IIP1
                  DO I=1,NWIN-1
                    JACOL(IFLD+I)=IIP1+I
                  END DO
                ELSEIF (NWIN == 0) THEN
                  IF (IFLDEXTRA(MAXEXTRAF) > 0) THEN
                    WRITE(OUTMESS,'("This data set has over ",i2," field labels which have not been encountered &
                    &before! Very creative but irritating! The field labelled ",A," has not been encountered before &
                    &and it and any other new fields after it will be ignored. If any of these fields are required see Kim")') &
                    &MAXEXTRAF,TRIM(TEMPSTR)
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LWARNLABEL=.FALSE.
                  ELSEIF (IEND < MAXFLDS) THEN
                    IEND=IEND+1
                    I=1
                    DO WHILE (IFLDEXTRA(I) >0)
                     I=I+1
                    END DO
                    IFLDEXTRA(I)=IFLD
                    LABEXTRA(I)=TRIM(KEYWORD)
                    JACOL(IFLD)=4000+I                !PUSH OUT OF DATABASE FIELD RANGE UNTIL WE KNOW HOW MANY WE HAVE
                  ELSEIF (.NOT. LTOOMANYF) THEN
                    WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                      &Extra fields will be ignored")')MAXFLDS
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LTOOMANYF=.TRUE.
                  END IF
                END IF
              CASE('M0':'M999')
                !THIS RANGE INCLUDES HISTORICAL ELECTRODE LOCATION LABELS TO ELIMINATE THEM FIRST.
                IF (TRIM(KEYWORD)=='MX') THEN
                  JACOL(IFLD)=ICHPH
                ELSEIF (TRIM(KEYWORD)=='M1X') THEN
                  !POTENTIAL ELECTRODE 1 X VALUE
                  JACOL(IFLD)=IP1X
                ELSEIF (TRIM(KEYWORD)=='MX1') THEN
                  !POTENTIAL ELECTRODE 1 X VALUE
                  JACOL(IFLD)=IP1X
                ELSEIF (TRIM(KEYWORD)=='M1Y') THEN
                  !POTENTIAL ELECTRODE 1 Y VALUE
                  JACOL(IFLD)=IP1Y
                ELSEIF (TRIM(KEYWORD)=='MY1') THEN
                  !POTENTIAL ELECTRODE 1 Y VALUE
                  JACOL(IFLD)=IP1Y
                ELSEIF (TRIM(KEYWORD)=='M1Z') THEN
                  !POTENTIAL ELECTRODE 1 Z VALUE
                  JACOL(IFLD)=IP1Z
                ELSEIF (TRIM(KEYWORD)=='M2X') THEN
                  !POTENTIAL ELECTRODE 2 X VALUE
                  JACOL(IFLD)=IP2X
                ELSEIF (TRIM(KEYWORD)=='MX2') THEN
                  !POTENTIAL ELECTRODE 2 X VALUE
                  JACOL(IFLD)=IP2X
                ELSEIF (TRIM(KEYWORD)=='M2Y') THEN
                  !POTENTIAL ELECTRODE 2 Y VALUE
                  JACOL(IFLD)=IP2Y
                ELSEIF (TRIM(KEYWORD)=='MY2') THEN
                  !POTENTIAL ELECTRODE 2 Y VALUE
                  JACOL(IFLD)=IP2Y
                ELSEIF (TRIM(KEYWORD)=='M2Z') THEN
                  !POTENTIAL ELECTRODE 2 Z VALUE
                  JACOL(IFLD)=IP2Z
                ELSEIF (.NOT. LFIRST .AND. NWIN > 0) THEN
                  !FIRST CHARGEABILITY WINDOW
                  LFIRST=.TRUE.
                  JACOL(IFLD)=IIP1
                  DO I=1,NWIN-1
                    JACOL(IFLD+I)=IIP1+I
                  END DO
                ELSEIF (NWIN == 0) THEN
                  IF (IFLDEXTRA(MAXEXTRAF) > 0) THEN
                    WRITE(OUTMESS,'("This data set has over ",i2," field labels which have not been encountered &
                    &before! Very creative but irritating! The field labelled ",A," has not been encountered before &
                    &and it and any other new fields after it will be ignored. If any of these fields are required see Kim")') &
                    &MAXEXTRAF,TRIM(TEMPSTR)
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LWARNLABEL=.FALSE.
                  ELSEIF (IEND < MAXFLDS) THEN
                    IEND=IEND+1
                    I=1
                    DO WHILE (IFLDEXTRA(I) >0)
                     I=I+1
                    END DO
                    IFLDEXTRA(I)=IFLD
                    LABEXTRA(I)=TRIM(KEYWORD)
                    JACOL(IFLD)=4000+I                !PUSH OUT OF DATABASE FIELD RANGE UNTIL WE KNOW HOW MANY WE HAVE
                  ELSEIF (.NOT. LTOOMANYF) THEN
                    WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                      &Extra fields will be ignored")')MAXFLDS
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LTOOMANYF=.TRUE.
                  END IF
                END IF
              CASE('ARES0':'ARES999')
                IF (.NOT. LFIRST .AND. NWIN > 0) THEN
                  !FIRST CHARGEABILITY WINDOW
                  LFIRST=.TRUE.
                  JACOL(IFLD)=IIP1
                  DO I=1,NWIN-1
                    JACOL(IFLD+I)=IIP1+I
                  END DO
                ELSEIF (NWIN == 0) THEN
                  IF (IFLDEXTRA(MAXEXTRAF) > 0) THEN
                    WRITE(OUTMESS,'("This data set has over ",i2," field labels which have not been encountered &
                    &before! Very creative but irritating! The field labelled ",A," has not been encountered before &
                    &and it and any other new fields after it will be ignored. If any of these fields are required see Kim")') &
                    &MAXEXTRAF,TRIM(TEMPSTR)
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LWARNLABEL=.FALSE.
                  ELSEIF (IEND < MAXFLDS) THEN
                    IEND=IEND+1
                    I=1
                    DO WHILE (IFLDEXTRA(I) >0)
                     I=I+1
                    END DO
                    IFLDEXTRA(I)=IFLD
                    LABEXTRA(I)=TRIM(KEYWORD)
                    JACOL(IFLD)=4000+I                !PUSH OUT OF DATABASE FIELD RANGE UNTIL WE KNOW HOW MANY WE HAVE
                  ELSEIF (.NOT. LTOOMANYF) THEN
                    WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                      &Extra fields will be ignored")')MAXFLDS
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LTOOMANYF=.TRUE.
                  END IF
                END IF
              CASE('CURRENT')
                JACOL(IFLD)=ICURR
              CASE('CURR')
                JACOL(IFLD)=ICURR
              CASE('I')
                JACOL(IFLD)=ICURR
              CASE('TXI')
                JACOL(IFLD)=ICURR
              CASE('LINE')
                !PLOT POINT LINE
                JACOL(IFLD)=ILINE
              CASE('BOREHOLE')
                !PLOT POINT LINE
                JACOL(IFLD)=ILINE
              CASE('HOLE')
                !PLOT POINT LINE
                JACOL(IFLD)=ILINE
              CASE('PROFILE')
                !PLOT POINT LINE
                JACOL(IFLD)=ILINE
              CASE('MAG0':'MAG999')
                IF (.NOT. LFIRST .AND. NHARM > 0) THEN
                  !FIRST HARMONIC
                  LFIRST=.TRUE.
                  JACOL(IFLD)=IIP1
                  DO I=1,(NHARM*2)-1
                    JACOL(IFLD+I)=IIP1+I
                  END DO
                ELSEIF (NHARM == 0) THEN
                  IF (IFLDEXTRA(MAXEXTRAF) > 0) THEN
                    WRITE(OUTMESS,'("This data set has over ",i2," field labels which have not been encountered &
                    &before! Very creative but irritating! The field labelled ",A," has not been encountered before &
                    &and it and any other new fields after it will be ignored. If any of these fields are required see Kim")') &
                    &MAXEXTRAF,TRIM(TEMPSTR)
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LWARNLABEL=.FALSE.
                  ELSEIF (IEND < MAXFLDS) THEN
                    IEND=IEND+1
                    I=1
                    DO WHILE (IFLDEXTRA(I) >0)
                     I=I+1
                    END DO
                    IFLDEXTRA(I)=IFLD
                    LABEXTRA(I)=TRIM(KEYWORD)
                    JACOL(IFLD)=4000+I                !PUSH OUT OF DATABASE FIELD RANGE UNTIL WE KNOW HOW MANY WE HAVE
                  ELSEIF (.NOT. LTOOMANYF) THEN
                    WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                      &Extra fields will be ignored")')MAXFLDS
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LTOOMANYF=.TRUE.
                  END IF
                END IF
              CASE('IP')
                JACOL(IFLD)=ICHPH
              CASE('CHT')
                JACOL(IFLD)=ICHPH
              CASE('NSPACE')
                JACOL(IFLD)=INVAL
              CASE('NLEVEL')
                JACOL(IFLD)=INVAL
              CASE('NSTACK')
                IF (IEND < MAXFLDS) THEN 
                  IEND=IEND+1
                  INSTAK=IEND
                  JACOL(IFLD)=INSTAK
                ELSEIF (.NOT. LTOOMANYF) THEN
                  WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                    &Extra fields will be ignored")')MAXFLDS
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  LTOOMANYF=.TRUE.
                END IF
              CASE('CYCLE')
                IF (IEND < MAXFLDS) THEN 
                  IEND=IEND+1
                  INSTAK=IEND
                  JACOL(IFLD)=INSTAK
                ELSEIF (.NOT. LTOOMANYF) THEN
                  WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                    &Extra fields will be ignored")')MAXFLDS
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  LTOOMANYF=.TRUE.
                END IF
              CASE('DUR')
                IF (IEND < MAXFLDS) THEN 
                  IEND=IEND+1
                  INSTAK=IEND
                  JACOL(IFLD)=INSTAK
                ELSEIF (.NOT. LTOOMANYF) THEN
                  WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                    &Extra fields will be ignored")')MAXFLDS
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  LTOOMANYF=.TRUE.
                END IF
              CASE('P1X')
                !POTENTIAL ELECTRODE 1 X VALUE
                JACOL(IFLD)=IP1X
              CASE('P1')
                !POTENTIAL ELECTRODE 1 X VALUE
                JACOL(IFLD)=IP1X
!              CASE('M1')           DISALLOW THIS AS IT CONFLICTS WITH CHANNEL WINDOW LABEL USAGE
!                !POTENTIAL ELECTRODE 1 X VALUE
!                JACOL(IFLD)=IP1X
              CASE('R1X')
                !POTENTIAL ELECTRODE 1 X VALUE
                JACOL(IFLD)=IP1X
              CASE('P1Y')
                !POTENTIAL ELECTRODE 1 Y VALUE
                JACOL(IFLD)=IP1Y
              CASE('R1Y')
                !POTENTIAL ELECTRODE 1 Y VALUE
                JACOL(IFLD)=IP1Y
              CASE('P1_LINE')
                !POTENTIAL ELECTRODE 1 Y VALUE
                JACOL(IFLD)=IP1Y
              CASE('P1Z')
                !POTENTIAL ELECTRODE 1 Z VALUE
                JACOL(IFLD)=IP1Z
              CASE('P1D')
                !POTENTIAL ELECTRODE 1 Z VALUE
                JACOL(IFLD)=IP1Z
              CASE('P2X')
                !POTENTIAL ELECTRODE 2 X VALUE
                JACOL(IFLD)=IP2X
              CASE('P2')
                !POTENTIAL ELECTRODE 2 X VALUE
                JACOL(IFLD)=IP2X
!              CASE('M2')
!                !POTENTIAL ELECTRODE 2 X VALUE
!                JACOL(IFLD)=IP2X
              CASE('R2X')
                !POTENTIAL ELECTRODE 2 X VALUE
                JACOL(IFLD)=IP2X
              CASE('P2Y')
                !POTENTIAL ELECTRODE 2 Y VALUE
                JACOL(IFLD)=IP2Y
              CASE('R2Y')
                !POTENTIAL ELECTRODE 2 Y VALUE
                JACOL(IFLD)=IP2Y
              CASE('P2_LINE')
                !POTENTIAL ELECTRODE 2 Y VALUE
                JACOL(IFLD)=IP2Y
              CASE('P2Z')
                !POTENTIAL ELECTRODE 2 Z VALUE
                JACOL(IFLD)=IP2Z
              CASE('P2D')
                !POTENTIAL ELECTRODE 2 Z VALUE
                JACOL(IFLD)=IP2Z
              CASE('R2Z')
                !POTENTIAL ELECTRODE 2 Z VALUE
                JACOL(IFLD)=IP2Z
              CASE('PLTPT')
                !PLOT POINT STATION
                JACOL(IFLD)=ISTAT
              CASE('STATION')
                !PLOT POINT STATION
                JACOL(IFLD)=ISTAT
              CASE('PLTX')
                !PLOT POINT STATION
                JACOL(IFLD)=IEAST
              CASE('PLTY')
                !PLOT POINT STATION
                JACOL(IFLD)=INRTH
              CASE('PLTZ')
                !PLOT POINT STATION
                JACOL(IFLD)=IELEV
              CASE('PLTEAST')
                !PLOT POINT STATION
                JACOL(IFLD)=IEAST
              CASE('PLTNORTH')
                !PLOT POINT STATION
                JACOL(IFLD)=INRTH
              CASE('PLTELEV')
                !PLOT POINT STATION
                JACOL(IFLD)=IELEV
              CASE('PH0':'PH999')       !LETS ASSUME THE FILE CREATOR EXPORTED MAG1,PH1,MAG2,PH2 ETC SO LFIRST WILL ALWAYS BE TRUE
                IF (.NOT. LFIRST .AND. NHARM > 0) THEN    !BUT JUST IN CASE - THIS WILL MEAN THE VARIABLES ARE ARSE ABOUT FACE IN THE DATABASE
                  !FIRST HARMONIC
                  LFIRST=.TRUE.
                  JACOL(IFLD)=IIP1
                  DO I=1,(NHARM*2)-1
                    JACOL(IFLD+I)=IIP1+I
                  END DO
                ELSEIF (NHARM == 0) THEN
                  IF (IFLDEXTRA(MAXEXTRAF) > 0) THEN
                    WRITE(OUTMESS,'("This data set has over ",i2," field labels which have not been encountered &
                    &before! Very creative but irritating! The field labelled ",A," has not been encountered before &
                    &and it and any other new fields after it will be ignored. If any of these fields are required see Kim")') &
                    &MAXEXTRAF,TRIM(TEMPSTR)
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LWARNLABEL=.FALSE.
                  ELSEIF (IEND < MAXFLDS) THEN
                    IEND=IEND+1
                    I=1
                    DO WHILE (IFLDEXTRA(I) >0)
                     I=I+1
                    END DO
                    IFLDEXTRA(I)=IFLD
                    LABEXTRA(I)=TRIM(KEYWORD)
                    JACOL(IFLD)=4000+I                !PUSH OUT OF DATABASE FIELD RANGE UNTIL WE KNOW HOW MANY WE HAVE
                  ELSEIF (.NOT. LTOOMANYF) THEN
                    WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                      &Extra fields will be ignored")')MAXFLDS
                    CALL WINFOBOX(OUTMESS,OUTSUB)
                    LTOOMANYF=.TRUE.
                  END IF
                END IF
              CASE('RES')
                JACOL(IFLD)=IARHO
              CASE('RHO')
                JACOL(IFLD)=IARHO
              CASE('RESIST')
                JACOL(IFLD)=IARHO
              CASE('APPRES')
                JACOL(IFLD)=IARHO
              CASE('RS')
                IF (IEND < MAXFLDS) THEN 
                  IEND=IEND+1
                  ICR=IEND
                  JACOL(IFLD)=ICR
                ELSEIF (.NOT. LTOOMANYF) THEN
                  WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                    &Extra fields will be ignored")')MAXFLDS
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  LTOOMANYF=.TRUE.
                END IF
              CASE('POT-RES')
                IF (IEND < MAXFLDS) THEN 
                  IEND=IEND+1
                  ICR=IEND
                  JACOL(IFLD)=ICR
                ELSEIF (.NOT. LTOOMANYF) THEN
                  WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                    &Extra fields will be ignored")')MAXFLDS
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  LTOOMANYF=.TRUE.
                END IF
              CASE('RXDIPOLE')
                IF (IEND < MAXFLDS) THEN 
                  IEND=IEND+1
                  IRXDIP=IEND
                  JACOL(IFLD)=IRXDIP
                  LABEL(IRXDIP)='RXDIPOLE'
                ELSEIF (.NOT. LTOOMANYF) THEN
                  WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                    &Extra fields will be ignored")')MAXFLDS
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  LTOOMANYF=.TRUE.
                END IF
              CASE('SD')
                IF (IEND < MAXFLDS) THEN 
                  IEND=IEND+1
                  ISTDDEV=IEND
                  JACOL(IFLD)=ISTDDEV
                ELSEIF (.NOT. LTOOMANYF) THEN
                  WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                    &Extra fields will be ignored")')MAXFLDS
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  LTOOMANYF=.TRUE.
                END IF
              CASE('STDDEV')
                IF (IEND < MAXFLDS) THEN 
                  IEND=IEND+1
                  ISTDDEV=IEND
                  JACOL(IFLD)=ISTDDEV
                ELSEIF (.NOT. LTOOMANYF) THEN
                  WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                    &Extra fields will be ignored")')MAXFLDS
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  LTOOMANYF=.TRUE.
                END IF
              CASE('SP')
                IF (IEND < MAXFLDS) THEN 
                  IEND=IEND+1
                  ISP=IEND
                  JACOL(IFLD)=ISP
                ELSEIF (.NOT. LTOOMANYF) THEN
                  WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                    &Extra fields will be ignored")')MAXFLDS
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  LTOOMANYF=.TRUE.
                END IF
              CASE('READING')
                !PLOT POINT STATION
                JACOL(IFLD)=ISTAT
              CASE('SOUNDING')
                !PLOT POINT STATION
                JACOL(IFLD)=ISTAT
              CASE('STN')
                !PLOT POINT STATION
                JACOL(IFLD)=ISTAT
              CASE('FID')
                !PLOT POINT STATION
                JACOL(IFLD)=ISTAT
              CASE('FIDUCIAL')
                !PLOT POINT STATION
                JACOL(IFLD)=ISTAT
              CASE('TXDIPOLE')
                IF (IEND < MAXFLDS) THEN 
                  IEND=IEND+1
                  ITXDIP=IEND
                  JACOL(IFLD)=ITXDIP
                  LABEL(ITXDIP)='TXDIPOLE'
                ELSEIF (.NOT. LTOOMANYF) THEN
                  WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                    &Extra fields will be ignored")')MAXFLDS
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  LTOOMANYF=.TRUE.
                END IF
              CASE('VP')
                JACOL(IFLD)=IPVOL
              CASE('MAG')
                JACOL(IFLD)=IPVOL
              CASE DEFAULT
                IF (IFLDEXTRA(MAXEXTRAF) > 0) THEN
                  WRITE(OUTMESS,'("This data set has over ",i2," field labels which have not been encountered &
                  &before! Very creative but irritating! The field labelled ",A," has not been encountered before &
                  &and it and any other new fields after it will be ignored. If any of these fields are required see Kim")') &
                  &MAXEXTRAF,TRIM(TEMPSTR)
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  LWARNLABEL=.FALSE.
                ELSEIF (IEND < MAXFLDS) THEN
                  IEND=IEND+1
                  I=1
                  DO WHILE (IFLDEXTRA(I) >0)
                   I=I+1
                  END DO
                  IFLDEXTRA(I)=IFLD
                  LABEXTRA(I)=TRIM(KEYWORD)
                  JACOL(IFLD)=4000+I                !PUSH OUT OF DATABASE FIELD RANGE UNTIL WE KNOW HOW MANY WE HAVE
                ELSEIF (.NOT. LTOOMANYF) THEN
                  WRITE(OUTMESS,'("The input file has more than ",I4," fields which is the current database limit. &
                    &Extra fields will be ignored")')MAXFLDS
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  LTOOMANYF=.TRUE.
                END IF
            END SELECT
          END DO
        END IF
    END SELECT
  END DO
  !
  ! set the Tx cycle for HSTR(12)
  IF (.NOT. LNULL(OFFTIME,ONTIME)) THEN
    WRITE(HSTR(12),'(F6.2," Seconds on ",F6.2," Seconds off")')ONTIME/1000.0,OFFTIME/1000.0
  ELSEIF (.NOT. LNULL(OFFTIME)) THEN                  !ASSUME THE SAME AS ONTIME
    WRITE(HSTR(12),'(F6.2," Seconds on ",F6.2," Seconds off")')OFFTIME/1000.0,OFFTIME/1000.0
  ELSEIF (.NOT. LNULL(ONTIME)) THEN                   !ASSUME THE SAME AS OFFTIME
    IF (TRIM(DTYPESTR) == 'TDIP') THEN
      WRITE(HSTR(12),'(F6.2," Seconds on ",F6.2," Seconds off")')ONTIME/1000.0,ONTIME/1000.0
    ELSEIF (TRIM(DTYPESTR) == 'FDIP') THEN
      WRITE(HSTR(12),'(F6.2," Seconds on ")')ONTIME/1000.0
    END IF
  ELSEIF (.NOT. LNULL(HARR(4))) THEN
    IF (TRIM(DTYPESTR) == 'TDIP') THEN
      WRITE(HSTR(12),'(F6.2," Seconds on ",F6.2," Seconds off")')1.0/(HARR(4)*4.0),1.0/(HARR(4)*4.0)
    ELSEIF (TRIM(DTYPESTR) == 'FDIP') THEN
      WRITE(HSTR(12),'(F6.2," Seconds on ",F6.2," Seconds off")')1.0/(HARR(4)*2.0),1.0/(HARR(4)*2.0)
    END IF
  END IF
  IF (IFLDEXTRA(1) > 0) THEN                !WE HAVE EXTRA UNDEFINED FIELDS - BRING BACK INTO THE DATABASE FIELD RANGE
    !GET THE NUMBER OF EXTRA FIELDS
    JEXTRA=1
    DO WHILE (IFLDEXTRA(JEXTRA) > 0)
      JEXTRA=JEXTRA+1
      IF (JEXTRA > MAXEXTRAF) EXIT
    END DO
    JEXTRA=JEXTRA-1
    DO I=1,MAXINFLDS
      IF (JACOL(I) > 4000) THEN
        JACOL(I)=JACOL(I)-4000+IEND-JEXTRA
      END IF
    END DO
  END IF
  IF (IEND == MAXFLDS) THEN
    NFLD=MAXFLDS
    IFLDUSEF=5                !USE THE PLOTPOINT ELEVATION FIELD FOR THE USE FLAG  
  ELSEIF(IEND > MAXFLDS) THEN
    WRITE(OUTMESS,'("Bugger me! The program has got somewhere it should never get, indicating a logic error. &
      &Get ready for a dummy spit. Report to Kim that IEND = ",I4)')IEND
    CALL WERRORBOX(OUTMESS,OUTSUB)
  ELSE
    NFLD=IEND+1
    IFLDUSEF=NFLD
  END IF 
  IF (JEXTRA > 0) THEN                !WE HAVE EXTRA UNDEFINED FIELDS
    DO I=1,JEXTRA
      LABEL(IEND-JEXTRA+I)=LABEXTRA(I)
    END DO
  END IF
  RETURN
  END SUBROUTINE RDESFHDR
  !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  SUBROUTINE READESF()
  !--------------------------------------------------------------------------------------
  !
  ! READS DATA BLOCK OF ESF FILE
  !
  !---------------------------------------------------------------------------------------
  !
  CHARACTER(LEN=5000) :: ALINE
  CHARACTER(LEN=20), PARAMETER :: OUTSUB ='READESF'
  CHARACTER(LEN=20) :: STR
  CHARACTER(LEN=255) :: ERRMESS
  CHARACTER(LEN=1),PARAMETER :: COLON = ':'
  CHARACTER(LEN=1),PARAMETER :: FORSLASH = '/'
  INTEGER :: IFLD
  INTEGER :: ICOL,IFLG
  INTEGER :: ILEN
  CHARACTER(LEN=1), PARAMETER :: SPACE = CHAR(32)
  CHARACTER(LEN=1), PARAMETER :: TAB = CHAR(9)

  ! SET MAX/MIN ARRAYS TO LARGE/SMALL VALUES
  ZMAX=ZNULL
  ZMIN=-ZNULL
  NPTS=0
  ! LOOP THRU INPUT FILE/WRITING TO BDB
  MAINLOOP: DO
    ICOL=1
    IFLG=0
    IFLD=0
    ZDATA=ZNULL
    ZDATA(ILINE)=RLINE
    READ(LUHIN,'(A)',IOSTAT=IERR)ALINE
    JLINE=JLINE+1
    SELECT CASE (IERR)
      CASE (:-1)
        EXIT
      CASE (1:)
        WRITE(OUTMESS,'("PROBLEM READING LINE ",I5," FROM DATA FILE")')JLINE
        CALL WERRORBOX(OUTMESS,OUTSUB)
      CASE (0)  ! NO ERROR - PROCESS LINE
        CALL IUPPERCASE(ALINE)
        ! REPLACE ANY TABS WITH A SPACE
        IF (INDEX(ALINE,TAB) /= 0) THEN
          CALL TXTMAC('*',.TRUE.,.FALSE.,TAB,SPACE,ALINE,ILEN)
        END IF
        IF (LEN_TRIM(ALINE) > 1) THEN
          DO
            CALL TXTEXT(SPACE,ALINE,ICOL,STR,IFLG)
            IF (IFLG > 0) EXIT
            IFLD=IFLD+1
            IF (TRIM(STR) == '*' .OR. TRIM(STR) == 'TN' .OR. INDEX(STR,'--') > 0) CYCLE ! CHECK FOR UNDEFINED OR TOO NOISY READING
            IF (JACOL(IFLD) < 0) CYCLE
            IF (JACOL(IFLD) > 0 ) THEN
              IF (ICHAR(STR(1:1)) > 57 .OR. ICHAR(STR(1:1)) < 43 )  THEN  !WE HAVE TEXT
                IF (LWARNPARSE) THEN
                  WRITE (OUTMESS,'("The following text has been encountered at line ",I6,&
                   &" ",A," when trying to read ",A," future text fields will not be flagged, &
                   &just set to null. Clean up the file if this is a problem")')JLINE,TRIM(STR),&
                   &TRIM(LABEXTRA(JACOL(IFLD)-IEND+JEXTRA))
                  CALL WINFOBOX(OUTMESS,OUTSUB)
                  LWARNPARSE=.FALSE.
                END IF
              ELSE
                READ(STR,*,IOSTAT=IERR,IOMSG=ERRMESS)ZDATA(JACOL(IFLD))
                IF (IERR /= 0) THEN
                  WRITE (OUTMESS,'("Problem parsing variable number ",I3," from ",A," at line ",I6,&
                  &" Clean up the file if this is not a header line")')IFLD,TRIM(STR),JLINE
                  CALL WERRORBOX(OUTMESS,OUTSUB)
                END IF
              END IF
            END IF
          END DO
          IF (LNULL(ZDATA(ISTAT))) THEN
            ZDATA(ISTAT)=NPTS+1
            IF (NPTS == 0) THEN
              OUTMESS='The file does not specify a station number so you will have to compute one from the &
              &electrode positions. The record number will be used as a temporary filler'
              CALL WINFOBOX(OUTMESS,OUTSUB)
            END IF
          END IF
          IF (LNULL(ZDATA(INVAL))) THEN
            ZDATA(INVAL)=1.0
            IF (NPTS == 0) THEN
              OUTMESS='The file does not specify an N value so you will have to compute one from the &
              &electrode positions. The number 1 will be used as a temporary filler'
              CALL WINFOBOX(OUTMESS,OUTSUB)
            END IF
          END IF
          IF (IFSTA > 0) THEN
            IF (.NOT. LNULL(ZDATA(ISTAT))) THEN
              ZDATA(IFSTA)=ZDATA(ISTAT)
            ELSEIF( .NOT. LNULL(ZDATA(IFSTA))) THEN
              ZDATA(ISTAT)=ZDATA(IFSTA)
            END IF
            IF (.NOT. LNULL(ZDATA(ILINE))) THEN
              ZDATA(IFLIN)=ZDATA(ILINE)
            ELSEIF( .NOT. LNULL(ZDATA(IFLIN))) THEN
              ZDATA(ILINE)=ZDATA(IFLIN)
            END IF
          END IF
          IF (TRIM(HSTR(10)) =='EW') THEN
            IF (LNULL(ZDATA(IC1Y))) THEN
              ZDATA(IC1Y)=RLINE
            END IF
            IF (LNULL(ZDATA(IC2Y))) THEN
              ZDATA(IC2Y)=ZDATA(IC1Y)
            END IF
            IF (TRIM(HSTR(6)) /= 'GRADIENT' .OR. .NOT. LNULL(HARR(14),HARR(17))) THEN
              IF (LNULL(ZDATA(IP1Y))) THEN
                ZDATA(IP1Y)=RLINE
              END IF
              IF (LNULL(ZDATA(IP2Y))) THEN
                ZDATA(IP2Y)=ZDATA(IP1Y)
              END IF
            END IF
          ELSEIF (TRIM(HSTR(10)) =='NS') THEN
            IF (LNULL(ZDATA(IC1X))) THEN
              ZDATA(IC1X)=RLINE
            END IF
            IF (LNULL(ZDATA(IC2X))) THEN
              ZDATA(IC2X)=RLINE
            END IF
            IF (TRIM(HSTR(6)) /= 'GRADIENT' .OR. .NOT. LNULL(HARR(13),HARR(16))) THEN
              IF (LNULL(ZDATA(IP1X))) THEN
                ZDATA(IP1X)=RLINE
              END IF
              IF (LNULL(ZDATA(IP2X))) THEN
                ZDATA(IP2X)=RLINE
              END IF
            END IF
          END IF
          IF (IRXDIP > 0 .AND. .NOT. LNULL(ZDATA(IRXDIP))) THEN
            IF (LNULL(HARR(12))) THEN
              HARR(12)=ZDATA(IRXDIP)
            ELSE
              HARR(12)=MIN(HARR(12),ZDATA(IRXDIP))
            ENDIF
          END IF
          IF (ITXDIP > 0 .AND. .NOT. LNULL(ZDATA(ITXDIP))) THEN
            IF (LNULL(HARR(12))) THEN
              HARR(12)=ZDATA(ITXDIP)
            ELSE
              HARR(12)=MIN(HARR(12),ZDATA(ITXDIP))
            ENDIF
          END IF
          ZDATA(IFLDUSEF)=1.0
          NPTS=NPTS+1
          CALL OUTRECBD()
        END IF
    END SELECT
  END DO MAINLOOP
  RETURN
  END SUBROUTINE READESF
  !-----------------------------------------------------------------------------------------
  SUBROUTINE CONSTTOVAL(KEYWORD,CONSTANT,RTEMP)
  !-----------------------------------------------------------------------------------------
  ! PARSE A NUMBER FROM A STRING
  !-----------------------------------------------------------------------------------------
  CHARACTER(LEN=*), INTENT(IN) :: KEYWORD
  CHARACTER(LEN=*), INTENT(IN) :: CONSTANT
  REAL, INTENT(OUT) :: RTEMP

  READ(CONSTANT,*,IOSTAT=IERR)RTEMP
  IF (IERR /=0) THEN
    WRITE(OUTMESS,'("Problem parseing keyword ",A," value from ",A)')TRIM(KEYWORD),TRIM(CONSTANT)
    CALL WERRORBOX(OUTMESS,OUTSUB)
  END IF  
  RETURN
  END SUBROUTINE CONSTTOVAL  
END SUBROUTINE ESFIPIN