SUBROUTINE OUTRECBD()
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!
! WRITES ONE RECORD TO BDB
!
! - changed write to a bufferred write to speed up large files - 10/12 - kmf
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
USE INPPARAMS
IMPLICIT NONE

INTEGER J

CALL BDBWRB(LUHOUT,NPTS,ZDATA,NFLD)
CALL BDSTATUS(NPTS,NPTS,ZNULL)
!
!   COLLECT MAX AND MINS
DO J=1,NFLD
  IF(.NOT. LNULL(ZDATA(J))) THEN
    ZMAX(J)=MAX(ZMAX(J),ZDATA(J))
    ZMIN(J)=MIN(ZMIN(J),ZDATA(J))
  END IF
END DO
RETURN
END SUBROUTINE OUTRECBD
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SUBROUTINE INITBDB(FILEOUT)
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!
!     SETS UP LABELS ETC FOR IP BDB
!
! KMF 12/00
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
USE INPPARAMS

IMPLICIT NONE
CHARACTER (LEN=255), INTENT(IN) :: FILEOUT      ! NAME FOR OUTPUT BDB

CHARACTER (LEN=7):: TIMESTR                     ! TEMPORARY STRING FOR TIME DOMAIN LABELS
INTEGER ::MODE,J
!
! GENERATE BDB NAME AND OPEN FILE
MODE=2
CALL BDBOPN(LUHOUT,MODE,FILEOUT)
!
!RESET ZMIN,ZMAX,LTYPE,DATA
LABEL(NFLD+1:MAXFLDS)='UNLABELED'
LTYPE=.TRUE.
LSORT=.FALSE.
ZMIN=-ZNULL
ZMAX=ZNULL
!
! set those LABELS THAT we know
LABEL(01)='LINE_NO.    '
LABEL(02)='STATION_NO. '
LABEL(03)='EASTING     '
LABEL(04)='NORTHING    '
LABEL(05)='ELEVATION   '
LABEL(06)='N_VALUE     '
LABEL(07)='CURRENT_Amps'
LABEL(08)='PVOLTAGE_mV '
LABEL(09)='ARHO_ohm-m  '
IF (LTDOMAIN) THEN
  LABEL(10)='CHARGE_mV/V '
  IF (LSPECTRAL) THEN
    DO J=1,NWIN
      IF (LZONGE) THEN
        ! THE FOLLOWING TIMES DIFFER SLIGHTLY FROM ZONGE'S BUT I DON'T FOLLOW
        ! THEIR LOGIC SO THESE WILL HAVE TO DO! MY LOGIC IS THAT THERE ARE 1024
        ! SAMPLES PER FULL WAVEFORM (512 IF FREQ = 1Htz). FIRST 7 SAMPLES AFTER Tx SWITCH OFF ARE
        ! IGNORED THEN EACH WINDOW BINS 19 CONSECUTIVE SAMPLES WITHOUT OVERLAP
        ! THUS FIRST WINDOW CENTRE IS 16.5 SAMPLES OUT FROM Tx SHUTDOWN AND EACH
        ! SAMPLE IS 1/(FREQ*1024) SECONDS WIDE
        IF (HARR(4) < 1.0) THEN
          WTIME(J)=(16.5+(J-1)*19.0)*1000.0/(HARR(4)*1024.0)
        ELSE
          WTIME(J)=(16.5+(J-1)*19.0)*1000.0/(HARR(4)*512.0)
        END IF
      END IF
      IF (WTIME(J).LT.10.0) THEN
        WRITE(TIMESTR,'(F7.5)') WTIME(J)
      ELSEIF  (WTIME(J).LT.100.0) THEN
        WRITE(TIMESTR,'(F7.4)') WTIME(J)
      ELSEIF  (WTIME(J).LT.1000.0) THEN
        WRITE(TIMESTR,'(F7.3)') WTIME(J)
      ELSEIF  (WTIME(J).LT.10000.0) THEN
        WRITE(TIMESTR,'(F7.2)') WTIME(J)
      ELSEIF  (WTIME(J).LT.100000.0) THEN
        WRITE(TIMESTR,'(F7.1)') WTIME(J)
      END IF
      DO WHILE (INDEX(TIMESTR,' ').NE.0)
        TIMESTR(INDEX(TIMESTR,' '):INDEX(TIMESTR,' '))='0'
      END DO
      LABEL(J+10)(1:5)='CH00-'
      IF (J.LT.10) THEN
        WRITE(LABEL(J+10)(4:4),'(I1)')J
      ELSE
        WRITE(LABEL(J+10)(3:4),'(I2)')J
      END IF
      LABEL(J+10)(6:12)=TIMESTR
    END DO
  END IF
ELSE
  IF (LSPECTRAL) THEN
    LABEL(10)='3PTDCPHASEmR'
    DO J=1,NHARM
      IF (J == 1) THEN
        WRITE(LABEL(11+(J-1)*2),'("H",I1,"_MAGNT_mV")')J-1
        WRITE(LABEL(11+(J-1)*2+1),'("H",I1,"_PHASEmRad")')J-1
      ELSEIF (J < 6) THEN
        WRITE(LABEL(11+(J-1)*2),'("H",I1,"_MAGNT_mV")')J*2-1
        WRITE(LABEL(11+(J-1)*2+1),'("H",I1,"_PHASEmRad")')J*2-1
      ELSEIF (J < 51) THEN
        WRITE(LABEL(11+(J-1)*2),'("H",I2,"_MAGNT_mV")')J*2-1
        WRITE(LABEL(11+(J-1)*2+1),'("H",I2,"_PHASmRad")')J*2-1
      ELSE
        WRITE(LABEL(11+(J-1)*2),'("H",I3,"_MAGN_mV")')J*2-1
        WRITE(LABEL(11+(J-1)*2+1),'("H",I3,"_PHAmRad")')J*2-1
      END IF
    END DO      
  ELSE
    LABEL(10)='PHASE_mR    '
  ENDIF
END IF
NPTS=0
LOCX=3
LOCY=4
LSORT=.FALSE.
MODE=2
CALL BDBHDR(LUHOUT,MODE,NFLD,LOCX,LOCY,NPTS,LSORT,LABEL,ZMIN,ZMAX,LTYPE)
!
! LOAD EXTENDED HEADER INFO
MODE=2
CALL BDBXHDR(LUHOUT,MODE,HARR,HSTR)

RETURN
END SUBROUTINE INITBDB
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SUBROUTINE GETLINEDIR(STRING,IDIR)
! ----------------------------------------------------------------------
!
! LOOKS AT THE NON-BLANK END OF STRING FOR A COMPASS HEADING
! RETURNS IDIR AS A MULTIPLIER TO ACCOUNT FOR LINE DIRECTIONS
! AND RETURNS STRING WITHOUT THE TRAILING COMPASS HEADING
! IDIR=1, 2 STATION OR LINE IS N OR E
! IDIR=-1, -2 STATION OR LINE IS S OR W
!
! KMF 5/03
!------------------------------------------------------------------------
IMPLICIT NONE
CHARACTER(LEN=*), INTENT(INOUT) :: STRING
INTEGER, INTENT(OUT) :: IDIR

INTEGER :: JLEN
CHARACTER(LEN=255) :: OUTMESS
CHARACTER(LEN=20), PARAMETER :: OUTSUB = 'GETLINEDIR'


JLEN=LEN_TRIM(STRING)
IF (STRING(JLEN:JLEN)=='S' .OR. STRING(JLEN:JLEN)=='s') THEN
  IDIR=-1
  STRING(JLEN:JLEN)=' '
ELSEIF (STRING(JLEN:JLEN)=='W' .OR. STRING(JLEN:JLEN)=='w') THEN
  IDIR=-2
  STRING(JLEN:JLEN)=' '
ELSEIF (STRING(JLEN:JLEN)=='N' .OR. STRING(JLEN:JLEN)=='n') THEN
  IDIR=1
  STRING(JLEN:JLEN)=' '
ELSEIF (STRING(JLEN:JLEN)=='E' .OR. STRING(JLEN:JLEN)=='e') THEN
  IDIR=2
  STRING(JLEN:JLEN)=' '
ELSEIF (ICHAR(STRING(JLEN:JLEN)) > 47 .AND. ICHAR(STRING(JLEN:JLEN)) < 57 .OR. &
        ICHAR(STRING(JLEN:JLEN)) ==46 ) THEN
  IDIR=1
ELSE
  WRITE(OUTMESS,'("CANNOT RECOVER A COMPASS HEADING FROM LINE/STATION :",A)')TRIM(STRING)
  CALL WERRORBOX(OUTMESS,OUTSUB)
END IF
RETURN
END SUBROUTINE GETLINEDIR
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SUBROUTINE GETDIRECTION(INSTR,REALOUT,DIRSTR)
!----------------------------------------------------------------------------
!
! REMOVES ANY TRAILING DIRECTION INDICATOR FROM A LINE OR STATION NUMBER
! AND RETURNS A REAL NUMBER AND A DIRECTION STRING
!
! KMF 1/06
!- added trap for no numbers in INSTR - probably indicates a file corruption - kmf 3/11
!----------------------------------------------------------------------------
USE WINTERACTER

IMPLICIT NONE
CHARACTER(LEN=*), INTENT(INOUT) :: INSTR
REAL, INTENT(OUT) :: REALOUT
CHARACTER(LEN=2), INTENT(OUT) :: DIRSTR

INTEGER :: ILEN
INTEGER :: IDIR
INTEGER :: I

REAL, PARAMETER :: ZNULL = -1.0E30

CALL IUPPERCASE(INSTR)
ILEN=LEN_TRIM(INSTR)
IDIR=1
DIRSTR=''
IF (INSTR(ILEN:ILEN)=='S') THEN
  IDIR=-1
  DIRSTR='EW'
ELSEIF (INSTR(ILEN:ILEN)=='W') THEN
  IDIR=-2
  DIRSTR='NS'
ELSEIF (INSTR(ILEN:ILEN)=='N') THEN
  IDIR=1
  DIRSTR='EW'
ELSEIF (INSTR(ILEN:ILEN)=='E') THEN
  IDIR=2
  DIRSTR='NS'
END IF
!NOW REMOVE ANY  DIRECTION INDICATORS OR ALPHA CHARACTERS
DO I=1,ILEN
  IF (IACHAR(INSTR(I:I)).GT. 57 .OR. IACHAR(INSTR(I:I)).LT. 48) INSTR(I:I)=' '
END DO
IF (LEN_TRIM(INSTR) > 0 ) THEN
  READ(INSTR,*)REALOUT
  REALOUT=SIGN(REALOUT,REAL(IDIR))
ELSE
  REALOUT=ZNULL
END IF
RETURN
END SUBROUTINE GETDIRECTION
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SUBROUTINE RWCHANFL(CHFILE,NCHAN,WTIME,OFFTIME,MODE)
!-----------------------------------------------------------------------
!
! subroutine to read/write a standard ESF channel file
!
! - 9/14 - kmf 
!-----------------------------------------------------------------------
USE WINTERACTER

IMPLICIT NONE
CHARACTER(LEN=255), INTENT(IN) :: CHFILE    !NAME OF CHANNEL FILE
INTEGER, INTENT(INOUT) :: NCHAN             !NUMBER OF WINDOWS - RESET IN THIS ROUTINE - TAKE CARE. SET TO 0 ON INPUT TO ALLOW RESET
REAL, DIMENSION(50), INTENT(INOUT) :: WTIME !WINDOW CENTRE TIMES - ARRAY SIZE MAY NEED CHANGING IF MAXTIMES CHANGES IN THE FUTURE
REAL, INTENT(INOUT) :: OFFTIME              !TRANSMITTER OFF TIME IF EQUAL TO ON TIME 
INTEGER, INTENT(IN) :: MODE                 !READ/WRITE MODE=1=>READ MODE=2=>WRITE

INTEGER :: LUH
INTEGER :: I
INTEGER :: IERR
INTEGER :: ICOL, IFLG
INTEGER :: JLINE
INTEGER :: NCHINFILE                        !NUMBER OF CHANNELS IN FILE. MAY BE DIFFERENT TO NCHAN
REAL :: RNUM1, RNUM2
CHARACTER(LEN=255) :: OUTERR
CHARACTER(LEN=500) :: OUTMESS
CHARACTER(LEN=100) :: ALINE
CHARACTER(LEN=10) :: POSSTR                 !POSITION TO OPEN FILE - WE WANT TO APPEND TO A .CON FILE
CHARACTER(LEN=20), PARAMETER :: OUTSUB = 'RWCHANFL'
CHARACTER(LEN=2), PARAMETER :: CRLF = CHAR(13) // CHAR(10)
LOGICAL :: LDELAY                           !FLAG FOR FILE TYPE.


CALL GETHANDL(LUH)
IF (MODE == 1) THEN   !READ A CHANNEL FILE
  !OPEN THE INPUT FILE
  OPEN(UNIT=LUH,FILE=CHFILE,ACCESS='SEQUENTIAL',FORM='FORMATTED',STATUS='OLD',IOSTAT=IERR,IOMSG=OUTERR)
  IF (IERR .NE. 0) THEN
    WRITE(OUTMESS,'("CAN NOT OPEN CHANNEL FILE ",A," ERROR = ",I6,A,A)')TRIM(CHFILE),IERR,CRLF,TRIM(OUTERR)
    CALL WERRORBOX(OUTMESS,OUTSUB)
  END IF
  ALINE=TRIM(CHFILE)                !USE ALINE AS A TEMPORARY COPY BECAUSE OF INTENT IN  
  CALL IUPPERCASE(ALINE)
  IF (INDEX(ALINE,'.CON') > 0) THEN
    DO
      READ(LUH,'(A)',IOSTAT=IERR,IOMSG=OUTERR)ALINE
      IF (IERR /=0 ) THEN
        WRITE(OUTMESS,'("Problem reading windows times from the channel file. Error: ",A)')TRIM(OUTERR)
        CALL WERRORBOX(OUTMESS,OUTSUB)
      END IF
      CALL IUPPERCASE(ALINE)
      IF (INDEX(ALINE,'CHANNELFILE') > 0) EXIT
    END DO
  END IF
  READ(LUH,'(A)',IOSTAT=IERR,IOMSG=OUTERR)ALINE         !DESCRIPTIVE HEADER
  IF (IERR .NE. 0) THEN
    WRITE(OUTMESS,'("PROBLEM READING FIRST LINE OF CHANNEL FILE ",A," ERROR = ",I6,A,A)')TRIM(CHFILE),IERR,CRLF,TRIM(OUTERR)
    CALL WERRORBOX(OUTMESS,OUTSUB)
  END IF
  READ(LUH,'(A)',IOSTAT=IERR,IOMSG=OUTERR)ALINE         !NUMBER OF CHANNELS AND OPTIONAL OFFTIME VALUE
  IF (IERR .NE. 0) THEN
    WRITE(OUTMESS,'("PROBLEM READING SECOND LINE OF CHANNEL FILE ",A," ERROR = ",I6,A,A)')TRIM(CHFILE),IERR,CRLF,TRIM(OUTERR)
    CALL WERRORBOX(OUTMESS,OUTSUB)
  END IF
  CALL IUPPERCASE(ALINE) 
  IF (INDEX(ALINE,'NUMTIMES') > 0 ) THEN
    ICOL=INDEX(ALINE,'NUMTIMES')+9
  ELSEIF (INDEX(ALINE,'NCH') > 0 ) THEN
    ICOL=INDEX(ALINE,'NCH')+4
  ELSEIF (INDEX(ALINE,'WINDOWS') > 0 ) THEN
    ICOL=INDEX(ALINE,'WINDOWS')+8
  ELSE
    WRITE(OUTMESS,'("NO VALID KEYWORD FOR NUMBER OF CHANNELS FOUND ON SECOND LINE OF CHANNEL FILE. IN: ",A)')TRIM(ALINE)
    CALL WERRORBOX(OUTMESS,OUTSUB)
  END IF
  CALL IExtNum(ICOL,ALINE,'\D',NCHINFILE,iflg)
  IF (IFLG > 0) THEN
    WRITE(OUTMESS,'("ERROR PARSING NUMBER OF CHANNELS FROM: ",A)')TRIM(ALINE)
    CALL WERRORBOX(OUTMESS,OUTSUB)
  END IF
  IF (NCHAN == 0) NCHAN=NCHINFILE
  ICOL=INDEX(ALINE,'OFFTIME')+8
  IF (ICOL > 1) THEN
   CALL SExtNum(ICOL,ALINE,'\D',OFFTIME,iflg)
   IF (IFLG > 0) THEN
     WRITE(OUTMESS,'("ERROR PARSING OFFTIME FROM: ",A)')TRIM(ALINE)
     CALL WERRORBOX(OUTMESS,OUTSUB)
   END IF
  END IF
  READ(LUH,'(A)',IOSTAT=IERR,IOMSG=OUTERR)ALINE         !DATA TYPE
  IF (IERR .NE. 0) THEN
    WRITE(OUTMESS,'("PROBLEM READING THIRD LINE OF CHANNEL FILE ",A," ERROR = ",I6,A,A)')TRIM(CHFILE),IERR,CRLF,TRIM(OUTERR)
    CALL WERRORBOX(OUTMESS,OUTSUB)
  END IF
  CALL IUPPERCASE(ALINE) 
  IF (INDEX(ALINE,'DELAY') > 0 .AND. INDEX(ALINE,'WIDTH') > 0) THEN
    LDELAY=.TRUE.
  ELSEIF (INDEX(ALINE,'TIMESSTART') > 0 .AND. INDEX(ALINE,'TIMESEND') > 0) THEN
    LDELAY=.FALSE.
  ELSE
    WRITE(OUTMESS,'("NO VALID KEYWORDS FOR DELAY & WIDTH OR TIMESSTART & TIMESEND FOUND ON THIRD &
      &LINE OF CHANNEL FILE. IN: ",A)')TRIM(ALINE)
    CALL WERRORBOX(OUTMESS,OUTSUB)
  END IF
  JLINE=4
  DO I=1,NCHINFILE
    READ(LUH,'(A)',IOSTAT=IERR,IOMSG=OUTERR)ALINE         !NUMBER OF CHANNELS AND OPTIONAL OFFTIME VALUE
    IF (IERR .NE. 0) THEN
      WRITE(OUTMESS,'("PROBLEM READING LINE ",I4," OF CHANNEL FILE. ERROR = ",I6,A,A)')JLINE,IERR,CRLF,TRIM(OUTERR)
      CALL WERRORBOX(OUTMESS,OUTSUB)
    END IF
    ICOL=1
    CALL SExtNum(ICOL,ALINE,'\D',RNUM1,iflg)
    IF (IFLG > 0) THEN
      WRITE(OUTMESS,'("ERROR PARSING FIRST NUMBER FROM LINE ",I4," WHICH READ: ",A)')JLINE,TRIM(ALINE)
      CALL WERRORBOX(OUTMESS,OUTSUB)
    END IF
    CALL SExtNum(ICOL,ALINE,'\D',RNUM2,iflg)
    IF (IFLG > 0 .AND. .NOT. LDELAY) THEN
      WRITE(OUTMESS,'("ERROR PARSING SECOND NUMBER FROM LINE ",I4," WHICH READ: ",A)')JLINE,TRIM(ALINE)
      CALL WERRORBOX(OUTMESS,OUTSUB)
    END IF
    IF (LDELAY) THEN
      WTIME(I)=RNUM1
    ELSE
      WTIME(I)=(RNUM1+RNUM2)/2.0
    END IF
  END DO  
ELSE                  !WRITE A CHANNEL FILE
  ALINE=TRIM(CHFILE)                !USE ALINE AS A TEMPORARY COPY BECAUSE OF INTENT IN  
  CALL IUPPERCASE(ALINE)
  IF (INDEX(ALINE,'.CON') > 0) THEN
    POSSTR='APPEND'
  ELSE
    POSSTR='ASIS'
  END IF
  !OPEN THE INPUT FILE
  OPEN(UNIT=LUH,FILE=CHFILE,ACCESS='SEQUENTIAL',FORM='FORMATTED',STATUS='UNKNOWN',POSITION=POSSTR, &
    &IOSTAT=IERR,IOMSG=OUTERR)
  IF (IERR .NE. 0) THEN
    WRITE(OUTMESS,'("CAN NOT OPEN CHANNEL FILE ",A," ERROR = ",I6,A,A)')TRIM(CHFILE),IERR,CRLF,TRIM(OUTERR)
    CALL WERRORBOX(OUTMESS,OUTSUB)
  END IF
  WRITE(LUH,'("CHANNEL FILE EXPORTED BY IPPROC")',IOSTAT=IERR,IOMSG=OUTERR)
  IF (IERR .NE. 0) THEN
    WRITE(OUTMESS,'("PROBLEM WRITING HEADER LINE TO CHANNEL FILE. ERROR = ",I6,A,A)')IERR,CRLF,TRIM(OUTERR)
    CALL WERRORBOX(OUTMESS,OUTSUB)
  END IF
  WRITE(LUH,'("NUMTIMES:",I3,"  OFFTIME:",F7.2)',IOSTAT=IERR,IOMSG=OUTERR)NCHAN,OFFTIME
  IF (IERR .NE. 0) THEN
    WRITE(OUTMESS,'("PROBLEM WRITING HEADER SECOND LINE TO CHANNEL FILE. ERROR = ",I6,A,A)')IERR,CRLF,TRIM(OUTERR)
    CALL WERRORBOX(OUTMESS,OUTSUB)
  END IF
  WRITE(LUH,'("DELAY   WIDTH")',IOSTAT=IERR,IOMSG=OUTERR)
  IF (IERR .NE. 0) THEN
    WRITE(OUTMESS,'("PROBLEM WRITING HEADER THIRD LINE TO CHANNEL FILE. ERROR = ",I6,A,A)')IERR,CRLF,TRIM(OUTERR)
    CALL WERRORBOX(OUTMESS,OUTSUB)
  END IF
  !ASSUME WINDOWS DO NOT OVERLAP AND WINDOW 1 IS THE SAME WIDTH AS WINDOW 2
  RNUM2=(WTIME(2)-WTIME(1))
  WRITE(LUH,'(2F9.3)',IOSTAT=IERR,IOMSG=OUTERR)WTIME(1),RNUM2
  IF (IERR .NE. 0) THEN
    WRITE(OUTMESS,'("PROBLEM WRITING WINDOW TIME 1 TO CHANNEL FILE. ERROR = ",I6,A,A)')IERR,CRLF,TRIM(OUTERR)
    CALL WERRORBOX(OUTMESS,OUTSUB)
  END IF
  DO I=2,NCHAN
    RNUM1=RNUM2/2.0
    RNUM2=(WTIME(I)-WTIME(I-1)-RNUM1)*2
    WRITE(LUH,'(2F9.3)',IOSTAT=IERR,IOMSG=OUTERR)WTIME(I),RNUM2
    IF (IERR .NE. 0) THEN
      WRITE(OUTMESS,'("PROBLEM WRITING WINDOW TIME ",I3," TO CHANNEL FILE. ERROR = ",I6,A,A)')I,IERR,CRLF,TRIM(OUTERR)
      CALL WERRORBOX(OUTMESS,OUTSUB)
    END IF
  END DO
END IF

CLOSE(LUH)
RETURN
END SUBROUTINE RWCHANFL
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SUBROUTINE RESETTIME(NWINDOWS)
!-----------------------------------------------------------------------------
!
!SETS OR RESETS CHANNEL LABELS
!
!------------------------------------------------------------------------------
USE INPPARAMS
IMPLICIT NONE
INTEGER, INTENT(IN) :: NWINDOWS

INTEGER :: J
CHARACTER(LEN=7) :: TIMESTR
CHARACTER(LEN=6) :: FMAT
CHARACTER(LEN=6) :: TSTR

TIMESTR=' '
DO J=1,NWINDOWS
  IF (.NOT. LNULL(WTIME(J))) THEN
!    IF (LNULL(TDELAY)) THEN
!      WTIME(J)=WTIME(J)-RXDELAY
!    ELSE
!      WTIME(J)=WTIME(J)+TDELAY-RXDELAY
!    END IF
    IF (ABS(WTIME(J)).LT.10.0) THEN
      IF (WTIME(J) > 0.0) THEN
        FMAT='(F7.5)'
      ELSE
        FMAT='(F7.4)'
      END IF
    ELSEIF (ABS(WTIME(J)).LT.100.0) THEN
      IF (WTIME(J) > 0.0) THEN
        FMAT='(F7.4)'
      ELSE
        FMAT='(F7.3)'
      END IF
    ELSEIF (ABS(WTIME(J)).LT.1000.0) THEN
      IF (WTIME(J) > 0.0) THEN
        FMAT='(F7.3)'
      ELSE
        FMAT='(F7.2)'
      END IF
    ELSEIF (ABS(WTIME(J)).LT.1000.0) THEN
      IF (WTIME(J) > 0.0) THEN
        FMAT='(F7.2)'
      ELSE
        FMAT='(F7.1)'
      END IF
    ELSEIF (ABS(WTIME(J)).LT.10000.0) THEN
      IF (WTIME(J) > 0.0) THEN
        FMAT='(F7.1)'
      ELSE
        FMAT='(F7.0)'
      END IF
    END IF
    WRITE(TIMESTR,FMAT) WTIME(J)
    DO WHILE (INDEX(TIMESTR,' ').NE.0)
      TIMESTR(INDEX(TIMESTR,' '):INDEX(TIMESTR,' '))='0'
    END DO
    IF (J < 10) THEN
      WRITE(TSTR,'("CH0",I1,"- ")')J
    ELSE
      WRITE(TSTR,'("CH",I2,"- ")')J
    END IF
    LABEL(J+10)=TSTR
    LABEL(J+10)(6:12)=TIMESTR
  END IF
END DO
RETURN
END SUBROUTINE RESETTIME
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
